# -*- coding: utf-8 -*-
import copy
from abjad.tools import indicatortools
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools import stringtools
from abjad.tools import systemtools
from abjad.tools.topleveltools import new
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class Instrument(AbjadValueObject):
    '''A musical instrument.
    '''

    ### CLASS VARIABLES ###

    _format_slot = 'opening'

    __slots__ = (
        '_allowable_clefs',
        '_default_scope',
        '_do_not_format',
        '_instrument_name',
        '_instrument_name_markup',
        '_is_primary_instrument',
        '_performer_names',
        '_pitch_range',
        '_short_instrument_name',
        '_short_instrument_name_markup',
        '_sounding_pitch_of_written_middle_c',
        '_starting_clefs',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        instrument_name=None,
        short_instrument_name=None,
        instrument_name_markup=None,
        short_instrument_name_markup=None,
        allowable_clefs=None,
        pitch_range=None,
        sounding_pitch_of_written_middle_c=None,
        ):
        from abjad.tools import instrumenttools
        from abjad.tools import scoretools
        self._do_not_format = False
        if instrument_name is not None:
            instrument_name = str(instrument_name)
        self._instrument_name = instrument_name
        if instrument_name_markup is not None:
            instrument_name_markup = markuptools.Markup(
                instrument_name_markup)
        self._instrument_name_markup = instrument_name_markup
        if short_instrument_name is not None:
            short_instrument_name = str(short_instrument_name)
        self._short_instrument_name = short_instrument_name
        if short_instrument_name_markup is not None:
            short_instrument_name_markup = markuptools.Markup(
                short_instrument_name_markup)
        self._short_instrument_name_markup = short_instrument_name_markup
        allowable_clefs = allowable_clefs or ['treble']
        allowable_clefs = instrumenttools.ClefList(allowable_clefs)
        self._allowable_clefs = allowable_clefs
        if isinstance(pitch_range, str):
            pitch_range = pitchtools.PitchRange(pitch_range)
        elif isinstance(pitch_range, pitchtools.PitchRange):
            pitch_range = copy.copy(pitch_range)
        elif pitch_range is None:
            pitch_range = pitchtools.PitchRange()
        else:
            raise TypeError(pitch_range)
        self._pitch_range = pitch_range
        sounding_pitch_of_written_middle_c = \
            sounding_pitch_of_written_middle_c or pitchtools.NamedPitch("c'")
        sounding_pitch_of_written_middle_c = \
            pitchtools.NamedPitch(sounding_pitch_of_written_middle_c)
        self._sounding_pitch_of_written_middle_c = \
            sounding_pitch_of_written_middle_c
        self._default_scope = scoretools.Staff
        self._is_primary_instrument = False
        self._performer_names = ['instrumentalist']
        self._starting_clefs = copy.copy(allowable_clefs)

    ### PRIVATE PROPERTIES ###

    @staticmethod
    def _default_instrument_name_to_instrument_class(default_instrument_name):
        for instrument_class in Instrument._list_instruments():
            instrument = instrument_class()
            if instrument.instrument_name == default_instrument_name:
                return instrument_class

    @property
    def _scope_name(self):
        if isinstance(self._default_scope, type):
            return self._default_scope.__name__
        elif isinstance(self._default_scope, str):
            return self._default_scope
        else:
            return type(self._default_scope).__name__

    ### PRIVATE METHODS ###

    def _get_default_performer_name(self):
        if self._performer_names is None:
            performer_name = '{} player'.format(self._default_instrument_name)
            return performer_name
        else:
            return self._performer_names[-1]

    def _get_format_specification(self):
        return systemtools.FormatSpecification(
            self,
            repr_args_values=[],
            repr_is_indented=False,
            repr_kwargs_names=[],
            )

    # TODO: _scope_name needs to be taken from IndicatorExpression!
    #       should not be stored on instrument.
    def _get_lilypond_format(self):
        result = []
        if self._do_not_format:
            return result
        instrument_name_markup = self.instrument_name_markup
        if instrument_name_markup.direction is not None:
            instrument_name_markup = new(
                instrument_name_markup,
                direction=None,
                )
        line = r'\set {!s}.instrumentName = {!s}'
        line = line.format(
            self._scope_name,
            instrument_name_markup,
            )
        result.append(line)
        line = r'\set {!s}.shortInstrumentName = {!s}'
        short_instrument_name_markup = self.short_instrument_name_markup
        if short_instrument_name_markup.direction is not None:
            short_instrument_name_markup = new(
                short_instrument_name_markup,
                direction=None,
                )
        line = line.format(
            self._scope_name,
            short_instrument_name_markup,
            )
        result.append(line)
        return result

    def _get_performer_names(self):
        if self._performer_names is None:
            performer_name = '{} player'.format(self._default_instrument_name)
            return [performer_name]
        else:
            return self._performer_names[:]

    def _initialize_default_name_markups(self):
        if self._instrument_name_markup is None:
            if self.instrument_name:
                string = self.instrument_name
                string = stringtools.capitalize_start(string)
                markup = markuptools.Markup(contents=string)
                self._instrument_name_markup = markup
            else:
                self._instrument_name_markup = None
        if self._short_instrument_name_markup is None:
            if self.short_instrument_name:
                string = self.short_instrument_name
                string = stringtools.capitalize_start(string)
                markup = markuptools.Markup(contents=string)
                self._short_instrument_name_markup = markup
            else:
                self._short_instrument_name_markup = None

    @classmethod
    def _list_instrument_names(cls):
        r'''Lists instrument names.

        ::

            >>> function = instrumenttools.Instrument._list_instrument_names
            >>> for instrument_name in function():
            ...     instrument_name
            ...
            'accordion'
            'alto'
            'alto flute'
            ...

        Returns list.
        '''
        instrument_names = []
        for instrument_class in cls._list_instruments():
            instrument = instrument_class()
            assert instrument.instrument_name is not None, repr(instrument)
            instrument_names.append(instrument.instrument_name)
        instrument_names.sort(key=lambda x: x.lower())
        return instrument_names

    @staticmethod
    def _list_instruments(classes=None):
        r'''Lists instruments.

        ::

            >>> function = instrumenttools.Instrument._list_instruments
            >>> for instrument in function():
            ...     instrument.__name__
            ...
            'Accordion'
            'AltoFlute'
            'AltoSaxophone'
            'AltoTrombone'
            'AltoVoice'
            'BaritoneSaxophone'
            ...

        Returns list.
        '''
        from abjad.tools import instrumenttools
        if classes is None:
            classes = (instrumenttools.Instrument,)
        instruments = []
        for value in instrumenttools.__dict__.values():
            try:
                if issubclass(value, classes):
                    if value is not instrumenttools.Instrument:
                        instruments.append(value)
            except TypeError:
                pass
        instruments.sort(key=lambda x: x.__name__.lower())
        return instruments

    @classmethod
    def _list_primary_instruments(cls):
        primary_instruments = []
        for instrument_class in cls._list_instruments():
            instrument = instrument_class()
            if instrument._is_primary_instrument:
                primary_instruments.append(instrument_class)
        return primary_instruments

    @classmethod
    def _list_secondary_instruments(cls):
        secondary_instruments = []
        for instrument_class in cls._list_instruments():
            instrument = instrument_class()
            if not instrument._is_primary_instrument:
                secondary_instruments.append(instrument_class)
        return secondary_instruments

    ### PUBLIC PROPERTIES ###

    @property
    def allowable_clefs(self):
        r'''Gets allowable clefs of instrument.

        Returns clef list.
        '''
        if self._allowable_clefs is None:
            self._allowable_clefs = instrumenttools.ClefList('treble')
        return self._allowable_clefs

    @property
    def instrument_name(self):
        r'''Gets instrument name.

        Returns string.
        '''
        return self._instrument_name

    @property
    def instrument_name_markup(self):
        r'''Gets instrument name markup.

        Returns markup.
        '''
        if self._instrument_name_markup is None:
            self._initialize_default_name_markups()
        if not isinstance(self._instrument_name_markup, markuptools.Markup):
            markup = markuptools.Markup(contents=self._instrument_name_markup)
            self._instrument_name_markup = markup
        if self._instrument_name_markup.contents != ('',):
            return self._instrument_name_markup

    @property
    def pitch_range(self):
        r'''Gets pitch range of instrument.

        Returns pitch range.
        '''
        return self._pitch_range

    @property
    def short_instrument_name(self):
        r'''Gets short instrument name.

        Returns string.
        '''
        return self._short_instrument_name

    @property
    def short_instrument_name_markup(self):
        r'''Gets short instrument name markup.

        Returns markup.
        '''
        if self._short_instrument_name_markup is None:
            self._initialize_default_name_markups()
        if not isinstance(
            self._short_instrument_name_markup, markuptools.Markup):
            markup = markuptools.Markup(
                contents=self._short_instrument_name_markup)
            self._short_instrument_name_markup = markup
        if self._short_instrument_name_markup.contents != ('',):
            return self._short_instrument_name_markup

    @property
    def sounding_pitch_of_written_middle_c(self):
        r'''Gets sounding pitch of written middle C.

        Returns named pitch.
        '''
        return self._sounding_pitch_of_written_middle_c

    ### PUBLIC METHODS ###

    def transpose_from_sounding_pitch_to_written_pitch(self, note_or_chord):
        r'''Transposes `note_or_chord` from sounding pitch of instrument to
        written pitch of instrument.

        Returns `note_or_chord` with adjusted pitches.
        '''
        from abjad.tools import scoretools
        sounding_pitch = self.sounding_pitch_of_written_middle_c
        interval = pitchtools.NamedPitch('C4') - sounding_pitch
        interval *= -1
        if isinstance(note_or_chord, scoretools.Note):
            pitch = note_or_chord.written_pitch
            pitch = interval.transpose(pitch)
            note_or_chord.written_pitch = pitch
        elif isinstance(note_or_chord, scoretools.Chord):
            pitches = [
                interval.transpose(pitch)
                for pitch in note_or_chord.written_pitches
                ]
            note_or_chord.written_pitches = pitches
        else:
            message = 'must be note or chord: {!r}.'
            message = message.format(note_or_chord)
            raise TypeError(message)

    def transpose_from_written_pitch_to_sounding_pitch(self, note_or_chord):
        r'''Transposes `argument` from written pitch of instrument to sounding
        pitch of instrument.

        Returns `note_or_chord` with adjusted pitches.
        '''
        from abjad.tools import scoretools
        sounding_pitch = self.sounding_pitch_of_written_middle_c
        interval = pitchtools.NamedPitch('C4') - sounding_pitch
        if isinstance(note_or_chord, scoretools.Note):
            pitch = note_or_chord.written_pitch
            pitch = interval.transpose(pitch)
            note_or_chord.written_pitch = pitch
        elif isinstance(note_or_chord, scoretools.Chord):
            pitches = [
                interval.transpose(pitch)
                for pitch in note_or_chord.written_pitches
                ]
            note_or_chord.written_pitches = pitches



# -*- coding: utf-8 -*-


def yield_all_partitions_of_integer(n):
    r'''Yields all partitions of positive integer `n` in descending lex order.

    ::

        >>> for partition in mathtools.yield_all_partitions_of_integer(7):
        ...     partition
        ...
        (7,)
        (6, 1)
        (5, 2)
        (5, 1, 1)
        (4, 3)
        (4, 2, 1)
        (4, 1, 1, 1)
        (3, 3, 1)
        (3, 2, 2)
        (3, 2, 1, 1)
        (3, 1, 1, 1, 1)
        (2, 2, 2, 1)
        (2, 2, 1, 1, 1)
        (2, 1, 1, 1, 1, 1)
        (1, 1, 1, 1, 1, 1, 1)

    Returns generator of positive integer tuples of length at least 1.
    '''
    from abjad.tools import mathtools
    if not isinstance(n, int):
        message = 'must be integer.'
        raise TypeError(message)
    if not 0 < n:
        message = 'must be positive.'
        raise ValueError(message)
    partition = (n,)
    while partition is not None:
        yield partition
        partition = mathtools.next_integer_partition(partition)



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.pitchtools import NumberedPitchClass


def test_pitchtools_NumberedPitchClass___repr___01():
    r'''Numbered pitch-class repr is evaluable.
    '''

    numbered_pitch_class_1 = pitchtools.NumberedPitchClass(1)
    numbered_pitch_class_2 = eval(repr(numbered_pitch_class_1))

    assert isinstance(numbered_pitch_class_1, pitchtools.NumberedPitchClass)
    assert isinstance(numbered_pitch_class_2, pitchtools.NumberedPitchClass)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_NoteHead___init___01():
    r'''Initialize note-head by number.
    '''

    notehead = scoretools.NoteHead(6)
    assert notehead.written_pitch == NamedPitch(6)


def test_scoretools_NoteHead___init___02():
    r'''Initialize note-head by LilyPond-style pitch string.
    '''

    notehead = scoretools.NoteHead('cs,,,')
    assert notehead.written_pitch == NamedPitch('cs,,,')


def test_scoretools_NoteHead___init___03():
    r'''Initialize note-head by other note-head instance.
    '''

    notehead = scoretools.NoteHead(6)
    new = scoretools.NoteHead(notehead)

    assert notehead is not new
    assert notehead.written_pitch.numbered_pitch._pitch_number == 6
    assert new.written_pitch.numbered_pitch._pitch_number == 6


def test_scoretools_NoteHead___init___04():
    r'''Initialize note-head with tweak pairs.
    '''

    note_head = scoretools.NoteHead("cs''", tweak_pairs=(('color', 'red'),))
    tweak = lilypondnametools.LilyPondNameManager()
    tweak.color = 'red'

    assert note_head.written_pitch == NamedPitch("cs''")
    assert note_head.tweak == tweak



# -*- coding: utf-8 -*-
from abjad.tools import markuptools
from abjad.tools.spannertools.Spanner import Spanner


class HorizontalBracketSpanner(Spanner):
    r'''Horizontal bracket spanner.

    ..  container:: example

        ::

            >>> voice = Voice("c'4 d'4 e'4 f'4")
            >>> voice.consists_commands.append('Horizontal_bracket_engraver')
            >>> spanner = spannertools.HorizontalBracketSpanner()
            >>> attach(spanner, voice[:])
            >>> show(voice) # doctest: +SKIP

        ..  doctest::

            >>> f(voice)
            \new Voice \with {
                \consists Horizontal_bracket_engraver
            } {
                c'4 \startGroup
                d'4
                e'4
                f'4 \stopGroup
            }

    Formats LilyPond ``\startGroup`` command on first leaf in spanner.

    Formats LilyPond ``\stopGroup`` command on last leaf in spanner.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_markup',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        overrides=None,
        markup=None,
        ):
        Spanner.__init__(
            self,
            overrides=overrides,
            )
        if markup is not None:
            markup = markuptools.Markup(markup)
        self._markup = markup

    ### PRIVATE METHODS ###

    def _format_right_of_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            result.append(r'\startGroup')
        if self._is_my_last_leaf(leaf):
            result.append(r'\stopGroup')
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def markup(self):
        r'''Gets horizonal bracket spanner markup.

        ..  container:: example

            Gets markup:

            ::

                >>> markup = Markup('3-1[012]').smaller()
                >>> spanner = spannertools.HorizontalBracketSpanner(
                ...     markup=markup,
                ...     )

            ::

                >>> spanner.markup
                Markup(contents=[MarkupCommand('smaller', '3-1[012]')])

        ..  container:: example

            Defaults to none:

            ::

                >>> spanner = spannertools.HorizontalBracketSpanner()
                >>> spanner.markup is None
                True

        Set to markup or none.

        Returns markup or none.
        '''
        return self._markup



# -*- encoding: utf-8 -*-
import argparse
import doctest
import re
import shutil
import unittest
from abjad.tools import commandlinetools
from abjad.tools import stringtools
from abjad.tools import systemtools
try:
    import pathlib
except ImportError:
    import pathlib2 as pathlib
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO


class TestCheckClassSections(unittest.TestCase):

    ansi_escape = re.compile(r'\x1b[^m]*m')
    test_path = pathlib.Path(__file__).parent
    temp_test_dir_name = 'temp_test_dir'
    subdirectory_path = test_path.joinpath(temp_test_dir_name)

    # Specific test case file contents
    test_bad_header_order_module_path = subdirectory_path.joinpath(
        'BadHeaderOrder.py'
        )
    test_property_in_methods_module_path = subdirectory_path.joinpath(
        'PropInMethods.py'
        )
    test_method_in_properties_module_path = subdirectory_path.joinpath(
        'MethodInProps.py'
        )
    test_multiple_errors_in_file_module_path = subdirectory_path.joinpath(
        'MultipleErrors.py'
        )
    test_multiple_classes_in_one_module_path = subdirectory_path.joinpath(
        'MultipleClasses.py'
        )
    test_passing_module_path = subdirectory_path.joinpath(
        'GoodClass.py'
        )
    test_non_property_decorators_module_path = subdirectory_path.joinpath(
        'NonPropertyDecorators.py'
        )
    test_bad_header_order_module_contents = stringtools.normalize(r'''
    class BadHeaders:
        ### CLASS VARIABLES ###
        ### CONSTRUCTOR ###
        ### INITIALIZER ###
        ### SPECIAL METHODS ###
        ### PRIVATE METHODS ###
        ### PUBLIC METHODS ###
        ### PUBLIC PROPERTIES ###
        ### PRIVATE PROPERTIES ###
        # ^ Properties are out of order
    ''')
    test_property_in_methods_module_contents = stringtools.normalize(r'''
    class PropertyInMethods:
        ### CLASS VARIABLES ###
        ### CONSTRUCTOR ###
        ### INITIALIZER ###
        ### SPECIAL METHODS ###
        ### PRIVATE METHODS ###
        ### PUBLIC METHODS ###
        @property
        def i_dont_belong_here(self):
            pass
        ### PRIVATE PROPERTIES ###
        ### PUBLIC PROPERTIES ###
    ''')
    test_method_in_properties_module_contents = stringtools.normalize(r'''
    class MethodInProperties:
        ### CLASS VARIABLES ###
        ### CONSTRUCTOR ###
        ### INITIALIZER ###
        ### SPECIAL METHODS ###
        ### PRIVATE METHODS ###
        ### PUBLIC METHODS ###
        ### PRIVATE PROPERTIES ###
        ### PUBLIC PROPERTIES ###
        def i_dont_belong_here(self):
            pass
    ''')
    test_multiple_errors_in_file_module_contents = stringtools.normalize(r'''
    class MultipleErrors:
        ### CLASS VARIABLES ###
        ### CONSTRUCTOR ###
        ### INITIALIZER ###
        ### SPECIAL METHODS ###
        ### PRIVATE METHODS ###
        ### PUBLIC METHODS ###
        @property
        def doesnt_belong_here(self):
            pass
        @property
        def also_doesnt_belong_here(self):
            pass
        ### PRIVATE PROPERTIES ###
        ### PUBLIC PROPERTIES ###
        def i_dont_belong_here(self):
            pass
    ''')
    test_multiple_classes_in_one_module_contents = stringtools.normalize(r'''
    class GoodClassOne:
        ### CLASS VARIABLES ###
        ### CONSTRUCTOR ###
        ### INITIALIZER ###
        ### SPECIAL METHODS ###
        ### PRIVATE METHODS ###
        ### PUBLIC METHODS ###
        ### PRIVATE PROPERTIES ###
        ### PUBLIC PROPERTIES ###
        pass
    class GoodClassTwo:
        ### CLASS VARIABLES ###
        ### CONSTRUCTOR ###
        ### INITIALIZER ###
        ### SPECIAL METHODS ###
        ### PRIVATE METHODS ###
        ### PUBLIC METHODS ###
        ### PRIVATE PROPERTIES ###
        ### PUBLIC PROPERTIES ###
        pass
    ''')
    test_non_property_decorators_module_contents = stringtools.normalize(r'''
    class NonPropertyDecoratorsInMethods:
        ### CLASS VARIABLES ###
        ### CONSTRUCTOR ###
        ### INITIALIZER ###
        ### SPECIAL METHODS ###
        ### PRIVATE METHODS ###
        ### PUBLIC METHODS ###
        @staticmethod
        def i_belong_here(self):
            pass
        @classmethod
        def me_too(self):
            pass
        @abc.abstractmethod
        def me_three(self):
            pass
        @lex.TOKEN()
        def me_four(self):
            pass
        ### PRIVATE PROPERTIES ###
        ### PUBLIC PROPERTIES ###
    ''')
    test_passing_module_contents = stringtools.normalize(r'''
    class PassingClass:
        ### CLASS VARIABLES ###
        x = 5
        ### CONSTRUCTOR ###
        ### INITIALIZER ###
        def __init__(self):
            pass
        ### SPECIAL METHODS ###
        def __str__(self):
            print("Hello, testing!")
        ### PRIVATE METHODS ###
        def _some_private_method(self):
            pass
        ### PUBLIC METHODS ###
        def some_public_method(self):
            pass
        ### PRIVATE PROPERTIES ###
        @property
        def _i_belong_here(self):
            pass
        @_i_belong_here.setter
        def _i_belong_here(self, value):
            pass
        ### PUBLIC PROPERTIES ###
        @property
        def i_belong_here(self):
            pass
        @i_belong_here.setter
        def i_belong_here(self, value):
            pass
    ''')

    ### TEST HELPER METHODS ###

    def compare_strings(self, expected, actual):
        example = argparse.Namespace()
        example.want = expected
        output_checker = doctest.OutputChecker()
        flags = (
            doctest.NORMALIZE_WHITESPACE |
            doctest.ELLIPSIS |
            doctest.REPORT_NDIFF
            )
        success = output_checker.check_output(expected, actual, flags)
        if not success:
            diff = output_checker.output_difference(example, actual, flags)
            raise Exception(diff)

    def tearDown(self):
        shutil.rmtree(str(self.subdirectory_path))
        self.string_io.close()

    def run_script_on_modules(
        self,
        modules,
        script_args=None,
        working_directory=None
        ):
        r'''Return the output and exit code of CheckClassSections
        when run against `modules`.

        `modules` should be a list of 2-tuples of strings in the form of
        (file_name, file_contents).

        `script_args` is an optional list of arguments to be passed to
        CheckClassSections.

        `working_directory` is an optional directory to run the script from.
        if none is passed, `self.test_path` is used.

        This method actually touches the file system, creating a temporary
        directory at `self.temp_test_dir_name` and creating temporary
        modules as specified in `modules`. `self.tearDown()` cleans up after
        this method.

        Returns a 2-tuple of (script_output, exit_code).
        '''
        # Handle arguments
        if script_args:
            if isinstance(script_args, list):
                command = [s_arg for s_arg in map(str, script_args)]
            else:
                command = [str(script_args)]
        else:
            command = []
        if working_directory:
            test_working_directory = working_directory
        else:
            test_working_directory = self.test_path
        # Create temporary testing modules
        if not self.subdirectory_path.exists():
            self.subdirectory_path.mkdir()
        for case in modules:
            with open(str(case[0]), 'w') as file_pointer:
                file_pointer.write(case[1])
        self.string_io = StringIO()
        # cd into test_working_directory and run the script with commands
        with systemtools.TemporaryDirectoryChange(str(test_working_directory)):
            with systemtools.RedirectedStreams(stdout=self.string_io):
                with self.assertRaises(SystemExit) as context_manager:
                    script = commandlinetools.CheckClassSections()
                    script(command)
        # Normalize script output for sane diffs
        script_output = self.ansi_escape.sub('', self.string_io.getvalue())
        script_output = stringtools.normalize(script_output)
        return (script_output, context_manager.exception.code)

    ### TEST CASES ###

    def test_bad_header_order(self):
        expected = stringtools.normalize('''
Recursively scanning {} for errors...
Errors in {}:
Lines [8]: BAD HEADER ORDER
===============================================================================
1 total files checked.
0 passed.
1 failed.
        '''.format(
                self.temp_test_dir_name,
                self.test_bad_header_order_module_path,
             )
        )
        test_modules = [
            (self.test_bad_header_order_module_path,
             self.test_bad_header_order_module_contents),
             ]
        # Run test
        script_output, exit_code = self.run_script_on_modules(
            test_modules, self.subdirectory_path
            )
        self.compare_strings(expected, script_output)
        self.assertEqual(exit_code, 1)

    def test_method_in_properties(self):
        expected = stringtools.normalize('''
Recursively scanning {} for errors...
Errors in {}:
Lines [9]: METHOD IN PROPERTIES SECTION
===============================================================================
1 total files checked.
0 passed.
1 failed.
        '''.format(
                self.temp_test_dir_name,
                self.test_method_in_properties_module_path,
             )
        )
        test_modules = [
            (self.test_method_in_properties_module_path,
             self.test_method_in_properties_module_contents),
            ]
        # Run test
        script_output, exit_code = self.run_script_on_modules(
            test_modules, self.subdirectory_path
            )
        self.compare_strings(expected, script_output)
        self.assertEqual(exit_code, 1)

    def test_property_in_methods(self):
        expected = stringtools.normalize('''
Recursively scanning {} for errors...
Errors in {}:
Lines [7]: PROPERTY IN METHODS SECTION
===============================================================================
1 total files checked.
0 passed.
1 failed.
        '''.format(
                self.temp_test_dir_name,
                self.test_property_in_methods_module_path,
            )
        )
        test_modules = [
            (self.test_property_in_methods_module_path,
             self.test_property_in_methods_module_contents),
             ]
        # Run test
        script_output, exit_code = self.run_script_on_modules(
            test_modules, self.subdirectory_path
            )
        self.compare_strings(expected, script_output)
        self.assertEqual(exit_code, 1)

    def test_multiple_errors_in_file(self):
        expected = stringtools.normalize('''
Recursively scanning {} for errors...
Errors in {}:
Lines [15]: METHOD IN PROPERTIES SECTION
Lines [7, 10]: PROPERTY IN METHODS SECTION
===============================================================================
1 total files checked.
0 passed.
1 failed.
        '''.format(
                self.temp_test_dir_name,
                self.test_multiple_errors_in_file_module_path,
            )
        )
        test_modules = [
            (self.test_multiple_errors_in_file_module_path,
             self.test_multiple_errors_in_file_module_contents),
             ]
        # Run test
        script_output, exit_code = self.run_script_on_modules(
            test_modules, self.subdirectory_path
            )
        self.compare_strings(expected, script_output)
        self.assertEqual(exit_code, 1)

    def test_non_property_decorators_in_methods_passes(self):
        expected = stringtools.normalize('''
Recursively scanning {} for errors...
1 total files checked.
1 passed.
0 failed.
        '''.format(
                self.temp_test_dir_name,
            )
        )
        test_modules = [
            (self.test_non_property_decorators_module_path,
             self.test_non_property_decorators_module_contents),
             ]
        # Run test
        script_output, exit_code = self.run_script_on_modules(
            test_modules, self.subdirectory_path
            )
        self.compare_strings(expected, script_output)
        self.assertEqual(exit_code, 0)

    def test_multiple_classes_in_one_module(self):
        expected = stringtools.normalize('''
Recursively scanning {} for errors...
1 total files checked.
1 passed.
0 failed.
        '''.format(self.temp_test_dir_name)
        )
        test_modules = [
            (self.test_multiple_classes_in_one_module_path,
             self.test_multiple_classes_in_one_module_contents)
            ]
        # Run test
        script_output, exit_code = self.run_script_on_modules(
            test_modules, self.subdirectory_path
            )
        self.compare_strings(expected, script_output)
        self.assertEqual(exit_code, 0)

    def test_passing_case(self):
        expected = stringtools.normalize('''
Recursively scanning {} for errors...
1 total files checked.
1 passed.
0 failed.
        '''.format(self.temp_test_dir_name)
        )
        test_modules = [
            (self.test_passing_module_path,
             self.test_passing_module_contents)
            ]
        # Run test
        script_output, exit_code = self.run_script_on_modules(
            test_modules, self.subdirectory_path
            )
        self.compare_strings(expected, script_output)
        self.assertEqual(exit_code, 0)

    def test_passing_case_without_passing_path(self):
        expected = stringtools.normalize('''
Recursively scanning current working directory for errors...
1 total files checked.
1 passed.
0 failed.
        '''.format(self.temp_test_dir_name)
        )
        test_modules = [
            (self.test_passing_module_path,
             self.test_passing_module_contents)
            ]
        # Run test
        script_output, exit_code = self.run_script_on_modules(
            test_modules,
            working_directory=self.subdirectory_path
            )
        self.compare_strings(expected, script_output)
        self.assertEqual(exit_code, 0)

    def test_passing_file_instead_of_dir(self):
        expected = stringtools.normalize('''
Scanning {} for errors...
1 total files checked.
1 passed.
0 failed.
        '''.format(self.test_passing_module_path)
        )
        test_modules = [
            (self.test_passing_module_path,
             self.test_passing_module_contents)
            ]
        # Run test
        script_output, exit_code = self.run_script_on_modules(
            test_modules,
            script_args=self.test_passing_module_path
            )
        self.compare_strings(expected, script_output)
        self.assertEqual(exit_code, 0)



# -*- coding: utf-8 -*-
from abjad import *
import pytest


def test_scoretools_Measure_append_01():
    r'''Time signature does not automatically adjust.
    '''

    measure = Measure((3, 4), "c' d' e'")
    measure.append('r')

    assert measure.is_overfull
    assert pytest.raises(Exception, 'f(measure)')


def test_scoretools_Measure_append_02():
    r'''Time signature adjusts automatically.
    '''

    measure = Measure((3, 4), "c' d' e'")
    measure.automatically_adjust_time_signature = True
    measure.append('r')

    assert not measure.is_misfilled
    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 4/4
            c'4
            d'4
            e'4
            r4
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_Octave___int___01():

    assert int(pitchtools.Octave('')) == 3
    assert int(pitchtools.Octave(',')) == 2
    assert int(pitchtools.Octave(',,')) == 1
    assert int(pitchtools.Octave(',,,')) == 0
    assert int(pitchtools.Octave("'")) == 4
    assert int(pitchtools.Octave("''")) == 5
    assert int(pitchtools.Octave("'''")) == 6



# -*- coding: utf-8 -*-


def iterate(client=None):
    r'''Makes iteration agent.

    ..  container:: example

        Example staff:

        ::

            >>> staff = Staff("c'4 e'4 d'4 f'4")
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4
                e'4
                d'4
                f'4
            }

    ..  container:: example

        Iterates staff by leaf pair:

            >>> for pair in iterate(staff).by_leaf_pair():
            ...     pair
            Selection([Note("c'4"), Note("e'4")])
            Selection([Note("e'4"), Note("d'4")])
            Selection([Note("d'4"), Note("f'4")])

    ..  container:: example

        Iterates staff by pitch:

            >>> for pitch in iterate(staff).by_pitch():
            ...     pitch
            NamedPitch("c'")
            NamedPitch("e'")
            NamedPitch("d'")
            NamedPitch("f'")

    ..  container:: example

        Returns iteration agent:

        ::

            >>> iterate(staff)
            IterationAgent(client=Staff("c'4 e'4 d'4 f'4"))

    '''
    from abjad.tools import agenttools
    from abjad.tools import expressiontools
    if client is not None:
        return agenttools.IterationAgent(client=client)
    expression = expressiontools.Expression()
    expression = expression.iterate()
    return expression



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import selectiontools
from abjad.tools.topleveltools import attach


def make_leaves_from_talea(
    talea,
    talea_denominator,
    decrease_durations_monotonically=True,
    forbidden_written_duration=None,
    spell_metrically=None,
    use_messiaen_style_ties=False,
    ):
    r'''Makes leaves from `talea`.

    Interprets positive elements in `talea` as notes numerators.

    Interprets negative elements in `talea` as rests numerators.

    Sets the pitch of all notes to middle C.

    ..  container:: example

        **Example 1.** Makes leaves from talea:

        ::

            >>> leaves = scoretools.make_leaves_from_talea([3, -3, 5, -5], 16)
            >>> staff = Staff(leaves)
            >>> staff.context_name = 'RhythmicStaff'
            >>> time_signature = TimeSignature((4, 4))
            >>> attach(time_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new RhythmicStaff {
                \time 4/4
                c'8.
                r8.
                c'4 ~
                c'16
                r4
                r16
            }

    ..  container:: example

        **Example 2.** Increases durations monotonically:

        ::

            >>> leaves = scoretools.make_leaves_from_talea(
            ...     [3, -3, 5, -5], 16,
            ...     decrease_durations_monotonically=False,
            ...     )
            >>> staff = Staff(leaves)
            >>> staff.context_name = 'RhythmicStaff'
            >>> time_signature = TimeSignature((4, 4))
            >>> attach(time_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new RhythmicStaff {
                \time 4/4
                c'8.
                r8.
                c'16 ~
                c'4
                r16
                r4
            }

    ..  container:: example

        **Example 3.** Forbids written durations greater than or equal
        to a half note:

        ::

            >>> leaves = scoretools.make_leaves_from_talea(
            ...     [3, -3, 5, -5], 16,
            ...     forbidden_written_duration=Duration(1, 4),
            ...     )
            >>> staff = Staff(leaves)
            >>> staff.context_name = 'RhythmicStaff'
            >>> time_signature = TimeSignature((4, 4))
            >>> attach(time_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new RhythmicStaff {
                \time 4/4
                c'8.
                r8.
                c'8 ~
                c'8 ~
                c'16
                r8
                r8
                r16
            }

    ..  container:: example

        **Example 4.** Spells unassignable durations metrically:

        ::

            >>> leaves = scoretools.make_leaves_from_talea(
            ...     [3, -3, 5, -5], 16,
            ...     spell_metrically='unassignable',
            ...     )
            >>> staff = Staff(leaves)
            >>> staff.context_name = 'RhythmicStaff'
            >>> time_signature = TimeSignature((4, 4))
            >>> attach(time_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new RhythmicStaff {
                \time 4/4
                c'8.
                r8.
                c'8. ~
                c'8
                r8.
                r8
            }

    ..  container:: example

        **Example 5.** Uses Messiaen-style ties:

        ::

            >>> leaves = scoretools.make_leaves_from_talea(
            ...     [5, 9], 8,
            ...     spell_metrically='unassignable',
            ...     use_messiaen_style_ties=True,
            ...     )
            >>> staff = Staff(leaves)
            >>> staff.context_name = 'RhythmicStaff'
            >>> time_signature = TimeSignature((4, 4))
            >>> attach(time_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new RhythmicStaff {
                \time 4/4
                c'4.
                c'4 \repeatTie
                c'4.
                c'4. \repeatTie
                c'4. \repeatTie
            }

    Returns selection.
    '''
    from abjad.tools import metertools
    from abjad.tools import scoretools
    from abjad.tools import spannertools

    assert all(x != 0 for x in talea), repr(talea)

    result = []
    for note_value in talea:
        if 0 < note_value:
            pitches = [0]
        else:
            pitches = [None]
        division = durationtools.Duration(
            abs(note_value),
            talea_denominator,
            )
        if (spell_metrically is True or
            (spell_metrically == 'unassignable' and
            not mathtools.is_assignable_integer(division.numerator))):
            meter = metertools.Meter(division)
            rhythm_tree_container = meter.root_node
            durations = [_.duration for _ in rhythm_tree_container]
        else:
            durations = [division]
        leaves = scoretools.make_leaves(
            pitches,
            durations,
            decrease_durations_monotonically=decrease_durations_monotonically,
            forbidden_written_duration=forbidden_written_duration,
            use_messiaen_style_ties=use_messiaen_style_ties,
            )
        if (
            1 < len(leaves) and
            not leaves[0]._has_spanner(spannertools.Tie) and
            not isinstance(leaves[0], scoretools.Rest)
            ):
            tie = spannertools.Tie(
                use_messiaen_style_ties=use_messiaen_style_ties,
                )
            attach(tie, leaves[:])
        result.extend(leaves)
    result = selectiontools.Selection(result)
    return result



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_BendAfter___init___01():
    r'''Initialize bend from bend amount.
    '''

    bend = indicatortools.BendAfter(16)
    assert isinstance(bend, indicatortools.BendAfter)


def test_indicatortools_BendAfter___init___02():
    r'''Initialize bend from other bend.
    '''

    bend_1 = indicatortools.BendAfter(16)
    bend_2 = indicatortools.BendAfter(bend_1)

    assert isinstance(bend_1, indicatortools.BendAfter)
    assert isinstance(bend_2, indicatortools.BendAfter)
    assert bend_1 == bend_2
    assert bend_1 is not bend_2



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools.topleveltools import iterate


def fill_measures_with_repeated_notes(
    argument, written_duration, iterctrl=None):
    '''Fill measures in `argument` with repeated notes.
    '''
    from abjad.tools import scoretools

    if iterctrl is None:
        iterctrl = lambda measure, i: True
    written_duration = durationtools.Duration(written_duration)
    for i, measure in enumerate(iterate(argument).by_class(scoretools.Measure)):
        if iterctrl(measure, i):
            time_signature = measure.time_signature
            total_duration = time_signature.duration
            prolation = time_signature.implied_prolation
            notes = scoretools.make_repeated_notes_with_shorter_notes_at_end(
                0, written_duration, total_duration, prolation)
            measure[:] = notes



# -*- coding: utf-8 -*-
from abjad import *


def test_rhythmtreetools_parse_rtm_syntax_01():

    rtm = '(3 (1 (3 (1 (3 (1 (3 (1 1 1 1))))))))'
    result = rhythmtreetools.parse_rtm_syntax(rtm)

    assert format(result) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 3/4 {
            c'4
            \tweak text #tuplet-number::calc-fraction-text
            \times 3/4 {
                c'4
                \tweak text #tuplet-number::calc-fraction-text
                \times 3/4 {
                    c'4
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 3/4 {
                        c'4
                        c'4
                        c'4
                        c'4
                    }
                }
            }
        }
        '''
        )



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Rest___setattr___01():
    r'''Slots constrain rest attributes.
    '''

    rest = Rest((1, 4))

    assert pytest.raises(AttributeError, "rest.foo = 'bar'")



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_ChordInversion_extent_to_figured_bass_string_01():

    chord_inversion = tonalanalysistools.ChordInversion(0)
    assert chord_inversion.extent_to_figured_bass_string(5) == ''
    assert chord_inversion.extent_to_figured_bass_string(7) == '7'

    chord_inversion = tonalanalysistools.ChordInversion(1)
    assert chord_inversion.extent_to_figured_bass_string(5) == '6'
    assert chord_inversion.extent_to_figured_bass_string(7) == '6/5'

    chord_inversion = tonalanalysistools.ChordInversion(2)
    assert chord_inversion.extent_to_figured_bass_string(5) == '6/4'
    assert chord_inversion.extent_to_figured_bass_string(7) == '4/3'

    chord_inversion = tonalanalysistools.ChordInversion(3)
    assert chord_inversion.extent_to_figured_bass_string(7) == '4/2'



# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_QGrid___eq___01():
    a = quantizationtools.QGrid()
    b = quantizationtools.QGrid()
    assert format(a) == format(b)
    assert a != b


def test_quantizationtools_QGrid___eq___02():
    a = quantizationtools.QGrid(
        root_node=quantizationtools.QGridContainer(
            preprolated_duration=1,
            children=[
                quantizationtools.QGridLeaf(
                    preprolated_duration=1,
                    q_event_proxies=[
                        quantizationtools.QEventProxy(
                            quantizationtools.SilentQEvent(100),
                            0.5,
                            ),
                        ],
                    ),
                ],
            ),
        next_downbeat=quantizationtools.QGridLeaf(
            preprolated_duration=1,
            q_event_proxies=[
                quantizationtools.QEventProxy(
                    quantizationtools.TerminalQEvent(200),
                    0.9,
                    ),
                ],
            ),
        )
    b = quantizationtools.QGrid(
        root_node=quantizationtools.QGridContainer(
            preprolated_duration=1,
            children=[
                quantizationtools.QGridLeaf(
                    preprolated_duration=1,
                    q_event_proxies=[
                        quantizationtools.QEventProxy(
                            quantizationtools.SilentQEvent(100),
                            0.5,
                            ),
                        ],
                    ),
                ],
            ),
        next_downbeat=quantizationtools.QGridLeaf(
            preprolated_duration=1,
            q_event_proxies=[
                quantizationtools.QEventProxy(
                    quantizationtools.TerminalQEvent(200),
                    0.9,
                    ),
                ],
            ),
        )
    assert format(a) == format(b)
    assert a != b


def test_quantizationtools_QGrid___eq___03():
    a = quantizationtools.QGrid()
    b = quantizationtools.QGrid(
        root_node=quantizationtools.QGridContainer(
            preprolated_duration=1,
            children=[
                quantizationtools.QGridLeaf(
                    preprolated_duration=1,
                    q_event_proxies=[
                        quantizationtools.QEventProxy(
                            quantizationtools.SilentQEvent(100),
                            0.5
                            )
                        ],
                    ),
                ],
            ),
        )
    c = quantizationtools.QGrid(
        next_downbeat=quantizationtools.QGridLeaf(
            preprolated_duration=1,
            q_event_proxies=[
                quantizationtools.QEventProxy(
                    quantizationtools.TerminalQEvent(200),
                    0.9
                    ),
                ],
            ),
        )
    d = quantizationtools.QGrid(
        root_node=quantizationtools.QGridContainer(
            preprolated_duration=1,
            children=[
                quantizationtools.QGridLeaf(
                    preprolated_duration=1,
                    q_event_proxies=[
                        quantizationtools.QEventProxy(
                            quantizationtools.SilentQEvent(100),
                            0.5
                            ),
                        ],
                    ),
                ],
            ),
        next_downbeat=quantizationtools.QGridLeaf(
            preprolated_duration=1,
            q_event_proxies=[
                quantizationtools.QEventProxy(
                    quantizationtools.TerminalQEvent(200),
                    0.9,
                    ),
                ],
            ),
        ),

    assert a != b
    assert a != c
    assert a != d
    assert b != c
    assert b != d
    assert c != d



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools.instrumenttools.Instrument import Instrument


class Contrabass(Instrument):
    r'''A contrabass.

    ::

        >>> staff = Staff("c'4 d'4 e'4 fs'4")
        >>> clef = Clef(name='bass')
        >>> attach(clef, staff)
        >>> contrabass = instrumenttools.Contrabass()
        >>> attach(contrabass, staff)
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> f(staff)
        \new Staff {
            \clef "bass"
            \set Staff.instrumentName = \markup { Contrabass }
            \set Staff.shortInstrumentName = \markup { Cb. }
            c'4
            d'4
            e'4
            fs'4
        }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_tuning',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        instrument_name='contrabass',
        short_instrument_name='cb.',
        instrument_name_markup=None,
        short_instrument_name_markup=None,
        allowable_clefs=('bass', 'treble'),
        default_tuning=('C1', 'A1', 'D2', 'G2'),
        pitch_range='[C1, G4]',
        sounding_pitch_of_written_middle_c='C3',
        ):
        Instrument.__init__(
            self,
            instrument_name=instrument_name,
            short_instrument_name=short_instrument_name,
            instrument_name_markup=instrument_name_markup,
            short_instrument_name_markup=short_instrument_name_markup,
            allowable_clefs=allowable_clefs,
            pitch_range=pitch_range,
            sounding_pitch_of_written_middle_c=\
                sounding_pitch_of_written_middle_c,
            )
        self._performer_names.extend([
            'string player',
            'contrabassist',
            'bassist',
            ])
        self._is_primary_instrument = True
        self._starting_clefs = type(self.allowable_clefs)(['bass'])
        self._default_tuning = indicatortools.Tuning(default_tuning)

    ### PUBLIC PROPERTIES ###

    @property
    def allowable_clefs(self):
        r'''Gets contrabass's allowable clefs.

        ..  container:: example

            ::

                >>> contrabass.allowable_clefs
                ClefList([Clef(name='bass'), Clef(name='treble')])

            ::

                >>> show(contrabass.allowable_clefs) # doctest: +SKIP

        Returns clef list.
        '''
        return Instrument.allowable_clefs.fget(self)

    @property
    def default_tuning(self):
        r'''Gets contrabass's default tuning.

        ..  container:: example

            >>> contrabass.default_tuning
            Tuning(pitches=PitchSegment(['c,,', 'a,,', 'd,', 'g,']))

        Returns tuning.
        '''
        return self._default_tuning

    @property
    def instrument_name(self):
        r'''Gets contrabass's name.

        ..  container:: example

            ::

                >>> contrabass.instrument_name
                'contrabass'

        Returns string.
        '''
        return Instrument.instrument_name.fget(self)

    @property
    def instrument_name_markup(self):
        r'''Gets contrabass's instrument name markup.

        ..  container:: example

            ::

                >>> contrabass.instrument_name_markup
                Markup(contents=['Contrabass'])

            ::

                >>> show(contrabass.instrument_name_markup) # doctest: +SKIP

        Returns markup.
        '''
        return Instrument.instrument_name_markup.fget(self)

    @property
    def pitch_range(self):
        r'''Gets contrabass's range.

        ..  container:: example

            ::

                >>> contrabass.pitch_range
                PitchRange(range_string='[C1, G4]')

            ::

                >>> show(contrabass.pitch_range) # doctest: +SKIP

        Returns pitch range.
        '''
        return Instrument.pitch_range.fget(self)

    @property
    def short_instrument_name(self):
        r'''Gets contrabass's short instrument name.

        ..  container:: example

            ::

                >>> contrabass.short_instrument_name
                'cb.'

        Returns string.
        '''
        return Instrument.short_instrument_name.fget(self)

    @property
    def short_instrument_name_markup(self):
        r'''Gets contrabass's short instrument name markup.

        ..  container:: example

            ::

                >>> contrabass.short_instrument_name_markup
                Markup(contents=['Cb.'])

            ::

                >>> show(contrabass.short_instrument_name_markup) # doctest: +SKIP

        Returns markup.
        '''
        return Instrument.short_instrument_name_markup.fget(self)

    @property
    def sounding_pitch_of_written_middle_c(self):
        r'''Gets sounding pitch of contrabass's written middle C.

        ..  container:: example

            ::

                >>> contrabass.sounding_pitch_of_written_middle_c
                NamedPitch('c')

            ::

                >>> show(contrabass.sounding_pitch_of_written_middle_c) # doctest: +SKIP

        Returns named pitch.
        '''
        return Instrument.sounding_pitch_of_written_middle_c.fget(self)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_GraceContainer_parentage_01():
    r'''Lone grace container carrier is none.
    '''

    gracecontainer = scoretools.GraceContainer("c'4 d'4 e'4 f'4")
    assert gracecontainer._carrier is None


def test_scoretools_GraceContainer_parentage_02():
    r'''Grace containers bound to leaf do have parent.
    '''

    note = Note(1, (1, 4))
    grace_container = scoretools.GraceContainer()
    attach(grace_container, note)
    grace_container = inspect_(note).get_grace_container()
    assert isinstance(grace_container, scoretools.GraceContainer)
    assert grace_container._carrier is note
    assert grace_container._carrier is note


def test_scoretools_GraceContainer_parentage_03():
    r'''Grace containers bound to leaf have their correct carrier
    after assignment.
    '''

    note = Note(1, (1, 4))
    after_grace = scoretools.GraceContainer([Note("e'16")], kind='after')
    attach(after_grace, note)
    grace = scoretools.GraceContainer([Note("e'16")], kind='grace')
    attach(grace, note)
    assert after_grace._carrier is note
    assert grace._carrier is note
    after_grace[:] = []
    notes = [Note("c'8"), Note("d'8")]
    after_grace.extend(notes)
    grace[:] = []
    notes = [Note("c'8"), Note("d'8")]
    grace.extend(notes)
    assert after_grace._carrier is note
    assert grace._carrier is note
    after_grace[:] = []
    grace[:] = []
    assert after_grace._carrier is note
    assert grace._carrier is note



# -*- coding: utf-8 -*-
from abjad import *


def test_schemetools_SchemePair_lilypond_format_01():
    r'''Right-hand side string forces quotes.
    '''

    scheme_pair = schemetools.SchemePair('font-name', 'Times')
    assert format(scheme_pair) == '#\'(font-name . "Times")'


def test_schemetools_SchemePair_lilypond_format_02():
    r'''Right-hand side nonstring does not force quotes.
    '''

    scheme_pair = schemetools.SchemePair('spacing', 4)
    assert format(scheme_pair) == "#'(spacing . 4)"



# -*- coding: utf-8 -*-
from abjad.tools.datastructuretools import TreeNode


class GraphvizTableHorizontalRule(TreeNode):
    r'''A Graphviz table horizontal rule.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Graphviz'

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        name=None,
        ):
        TreeNode.__init__(
            self,
            name=name,
            )

    ### SPECIAL METHODS ###

    def __str__(self):
        r'''Gets string representation of Graphviz table horizontal rule.

        Returns string.
        '''
        return '<HR/>'



# -*- coding: utf-8 -*-
from abjad.tools import patterntools
from abjad.tools import selectiontools
from abjad.tools.abctools import AbjadValueObject


class PatternedSelectorCallback(AbjadValueObject):
    r'''Patterned selector callback.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Callbacks'

    __slots__ = (
        '_apply_to_each',
        '_pattern',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        pattern=None,
        apply_to_each=None,
        ):
        if pattern is not None:
            assert isinstance(pattern, patterntools.Pattern)
        self._pattern = pattern
        if apply_to_each is not None:
            apply_to_each = bool(apply_to_each)
        self._apply_to_each = apply_to_each

    ### SPECIAL METHODS ###

    def __call__(self, argument, rotation=None):
        r'''Iterates tuple `argument`.

        Returns tuple in which each item is a selection or component.
        '''
        if rotation is None:
            rotation = 0
        rotation = int(rotation)
        if not self.pattern:
            return argument
        result = []
        if self.apply_to_each:
            for subexpr in argument:
                subresult = []
                length = len(subexpr)
                for index, item in enumerate(subexpr):
                    if self.pattern.matches_index(
                        index,
                        length,
                        rotation=rotation,
                        ):
                        subresult.append(item)
                if subresult:
                    subresult = selectiontools.Selection(subresult)
                    result.append(subresult)
        else:
            length = len(argument)
            for index, item in enumerate(argument):
                if self.pattern.matches_index(
                    index,
                    length,
                    rotation=rotation,
                    ):
                    result.append(item)
        return tuple(result)

    ### PUBLIC PROPERTIES ###

    @property
    def apply_to_each(self):
        r'''Is true if patterned selector callback is applied against each
        subexpression. Otherwise false.

        Returns true or false.
        '''
        return self._apply_to_each

    @property
    def pattern(self):
        r'''Gets pattern of patterned selector callback.

        Returns pattern.
        '''
        return self._pattern



# -*- coding: utf-8 -*-
from abjad import *


# TODO: rename to Measure_implicit_scaling or move there if already exists

def test_scoretools_Measure_special_prolation_01():
    r'''Measures with power-of-two time signature denominator
    contribute trivially to contents prolation.
    '''

    measure = Measure((4, 4), Note("c'4") * 4)

    assert measure[0].written_duration == Duration(1, 4)
    assert inspect_(measure[0]).get_duration() == Duration(1, 4)


def test_scoretools_Measure_special_prolation_02():
    r'''Measures with power-of-two time signature denominator
    contribute trivially to contents prolation.
    '''

    music = [
        Note("c'4"),
        scoretools.FixedDurationTuplet(Duration(2, 4), Note("c'4") * 3),
        Note("c'4"),
        ]
    measure = Measure((4, 4), music)
    leaves = select(measure).by_leaf()

    assert leaves[0].written_duration == Duration(1, 4)
    assert inspect_(leaves[0]).get_duration() == Duration(1, 4)
    assert leaves[1].written_duration == Duration(1, 4)
    assert inspect_(leaves[1]).get_duration() == Duration(1, 6)


def test_scoretools_Measure_special_prolation_03():
    r'''Measures with power-of-two time signature denominator
    contribute trivially to contents prolation.
    '''

    music = [
        Note("c'4"),
        scoretools.FixedDurationTuplet(Duration(2, 4), [
            scoretools.FixedDurationTuplet(Duration(2, 4), Note("c'4") * 3),
            Note("c'4")]),
        Note("c'4"),
        ]
    measure = Measure((4, 4), music)
    leaves = select(measure).by_leaf()

    assert leaves[0].written_duration == Duration(1, 4)
    assert inspect_(leaves[0]).get_duration() == Duration(1, 4)
    assert leaves[1].written_duration == Duration(1, 4)
    assert inspect_(leaves[1]).get_duration() == Duration(1, 9)


def test_scoretools_Measure_special_prolation_04():
    r'''Measures with power-of-two time signature denominators
    and implicit scaling scale the duration of their contents.
    '''

    measure = Measure((4, 5), "c'4 d'4 e'4 f'4")
    measure.implicit_scaling = True

    assert measure[0].written_duration == Duration(1, 4)
    assert inspect_(measure[0]).get_duration() == Duration(1, 5)


def test_scoretools_Measure_special_prolation_05():
    r'''Measures with power-of-two time signature denominators
    contribute nontrivially to prolation.
    '''

    music = [
        Note("c'4"),
        scoretools.FixedDurationTuplet(Duration(2, 4), Note("c'4") * 3),
        Note("c'4"),
        ]
    measure = Measure((4, 5), music)
    measure.implicit_scaling = True
    leaves = select(measure).by_leaf()

    assert leaves[0].written_duration == Duration(1, 4)
    assert inspect_(leaves[0]).get_duration() == Duration(1, 5)
    assert leaves[1].written_duration == Duration(1, 4)
    assert inspect_(leaves[1]).get_duration() == Duration(2, 15)


def test_scoretools_Measure_special_prolation_06():
    r'''Measures with power-of-two time signature denominators
    contribute nontrivially to prolation.
    '''

    music = [
        Note("c'4"),
        scoretools.FixedDurationTuplet(Duration(2, 4), [
            scoretools.FixedDurationTuplet(Duration(2, 4), Note("c'4") * 3),
            Note("c'4")]),
        Note("c'4"),
        ]
    measure = Measure((4, 5), music)
    measure.implicit_scaling = True
    leaves = select(measure).by_leaf()

    assert leaves[0].written_duration == Duration(1, 4)
    assert inspect_(leaves[0]).get_duration() == Duration(1, 5)
    assert leaves[1].written_duration == Duration(1, 4)
    assert inspect_(leaves[1]).get_duration() == Duration(4, 45)



# -*- coding: utf-8 -*-
import os


def play(argument):
    r'''Plays `argument`.

    ..  container:: example

        ::

            >>> note = Note("c'4")
            >>> play(note) # doctest: +SKIP

    Makes MIDI file.

    Appends ``.mid`` filename extension under Windows.

    Appends ``.midi`` filename extension under other operating systems. 

    Opens MIDI file.

    Returns none.
    '''
    from abjad import abjad_configuration
    from abjad.tools import systemtools
    from abjad.tools import topleveltools
    assert hasattr(argument, '__illustrate__')
    result = topleveltools.persist(argument).as_midi()
    midi_file_path, abjad_formatting_time, lilypond_rendering_time = result
    midi_player = abjad_configuration['midi_player']
    systemtools.IOManager.open_file(midi_file_path, midi_player)



# -*- coding: utf-8 -*-
from abjad import *


def test_datastructuretools_CyclicTuple___getslice___01():

    cyclic_tuple = datastructuretools.CyclicTuple(range(3))

    assert cyclic_tuple[:2] == (0, 1)
    assert cyclic_tuple[:10] == (0, 1, 2, 0, 1, 2, 0, 1, 2, 0)
    assert cyclic_tuple[2:10] == (2, 0, 1, 2, 0, 1, 2, 0)
    assert cyclic_tuple[1:-1] == (1,)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Measure__all_contents_are_scalable_by_same_multiplier_01():

    measure = Measure((1, 4), "c'4")
    assert measure._all_contents_are_scalable_by_multiplier(Multiplier(1, 2))


def test_scoretools_Measure__all_contents_are_scalable_by_same_multiplier_02():

    measure = Measure((1, 4), "c'4")
    assert measure._all_contents_are_scalable_by_multiplier(Multiplier(2, 1))


def test_scoretools_Measure__all_contents_are_scalable_by_same_multiplier_03():

    measure = Measure((1, 4), "c'4")
    assert not measure._all_contents_are_scalable_by_multiplier(
        Multiplier(2, 3))


def test_scoretools_Measure__all_contents_are_scalable_by_same_multiplier_04():

    measure = Measure((1, 4), "c'4")
    assert measure._all_contents_are_scalable_by_multiplier(Multiplier(3, 2))


def test_scoretools_Measure__all_contents_are_scalable_by_same_multiplier_05():

    measure = Measure((3, 16), "c'8.")
    assert measure._all_contents_are_scalable_by_multiplier(Multiplier(2, 3))


def test_scoretools_Measure__all_contents_are_scalable_by_same_multiplier_06():

    measure = Measure((3, 16), "c'8.")
    assert not measure._all_contents_are_scalable_by_multiplier(
        Multiplier(3, 2))



# -*- coding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class LilyPondContextSetting(AbjadObject):
    r'''LilyPond context setting.

    ::

        >>> context_setting = lilypondnametools.LilyPondContextSetting(
        ...    context_name='Score',
        ...    context_property='autoBeaming',
        ...    value=False,
        ...    )

    ::

        >>> print('\n'.join(context_setting.format_pieces))
        \set Score.autoBeaming = ##f

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_context_name',
        '_context_property',
        '_is_unset',
        '_value',
        )

    _format_leaf_children = False

    ### INITIALIZER ###

    def __init__(
        self,
        context_name=None,
        context_property='autoBeaming',
        is_unset=False,
        value=False,
        ):
        if context_name is not None:
            context_name = str(context_name)
        self._context_name = context_name
        assert isinstance(context_property, str) and context_property
        self._context_property = context_property
        if is_unset is not None:
            is_unset = bool(is_unset)
        self._is_unset = is_unset
        self._value = value

    ### SPECIAL METHODS ###

    def __eq__(self, argument):
        r'''Is true when `argument` is a LilyPond context setting with equivalent
        keyword values.
        '''
        from abjad.tools import systemtools
        return systemtools.TestManager.compare_objects(self, argument)

    def __hash__(self):
        r'''Hashes LilyPond context setting.

        Returns integer.
        '''
        from abjad.tools import systemtools
        hash_values = systemtools.StorageFormatAgent(self).get_hash_values()
        return hash(hash_values)

    ### PRIVATE METHODS ###

    def _get_lilypond_format_bundle(self, component=None):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        string = '\n'.join(self.format_pieces)
        lilypond_format_bundle.context_settings.append(string)
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def context_name(self):
        r'''Optional LilyPond context name.

        Returns string or none.
        '''
        return self._context_name

    @property
    def context_property(self):
        r'''LilyPond context property name.

        Returns string.
        '''
        return self._context_property

    @property
    def format_pieces(self):
        r'''Gets LilyPond context setting \set or \unset format pieces.

        Returns tuple of strings.
        '''
        from abjad.tools import schemetools
        result = []
        if not self.is_unset:
            result.append(r'\set')
        else:
            result.append(r'\unset')
        if self.context_name is not None:
            string = '{}.{}'.format(
                self.context_name,
                self.context_property,
                )
            result.append(string)
        else:
            result.append(self.context_property)
        result.append('=')
        value_pieces = schemetools.Scheme.format_embedded_scheme_value(
            self.value)
        value_pieces = value_pieces.split('\n')
        result.append(value_pieces[0])
        result[:] = [' '.join(result)]
        result.extend(value_pieces[1:])
        return tuple(result)

    @property
    def is_unset(self):
        r'''Is true if context setting unsets its value. Otherwise false.

        Returns boolean or none.
        '''
        return self._is_unset

    @property
    def value(self):
        r'''Value of LilyPond context setting.

        Returns arbitrary object.
        '''
        return self._value



# -*- coding: utf-8 -*-
from .AbjadObject import AbjadObject
from .AbjadValueObject import AbjadValueObject
from .ContextManager import ContextManager
from .Parser import Parser

_documentation_section = 'internals'



# -*- coding: utf-8 -*-
from abjad.tools import rhythmtreetools
import copy


def test_rhythmtreetools_RhythmTreeLeaf___copy___01():

    leaf = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=1)

    copied = copy.copy(leaf)

    assert format(leaf) == format(copied)
    assert leaf is not copied


def test_rhythmtreetools_RhythmTreeLeaf___copy___02():

    leaf = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=2, is_pitched=True)

    copied = copy.copy(leaf)

    assert format(leaf) == format(copied)
    assert leaf is not copied



# -*- coding: utf-8 -*-
from .AbjadConfiguration import AbjadConfiguration
from .BenchmarkScoreMaker import BenchmarkScoreMaker
from .Configuration import Configuration
from .FilesystemState import FilesystemState
from .ForbidUpdate import ForbidUpdate
from .FormatSpecification import FormatSpecification
from .IOManager import IOManager
from .ImportManager import ImportManager
from .LilyPondFormatBundle import LilyPondFormatBundle
from .LilyPondFormatManager import LilyPondFormatManager
from .NullContextManager import NullContextManager
from .ProgressIndicator import ProgressIndicator
from .RedirectedStreams import RedirectedStreams
from .StorageFormatAgent import StorageFormatAgent
from .StorageFormatSpecification import StorageFormatSpecification
from .TemporaryDirectory import TemporaryDirectory
from .TemporaryDirectoryChange import TemporaryDirectoryChange
from .TestManager import TestManager
from .TestCase import TestCase
from .Timer import Timer
from .UpdateManager import UpdateManager
from .WellformednessManager import WellformednessManager

_documentation_section = 'internals'



# -*- coding: utf-8 -*-
from abjad.tools import sequencetools
from abjad.tools.pitchtools.Vector import Vector


class IntervalClassVector(Vector):
    r'''Interval-class vector.

    ..  container:: example

        An interval-class vector:

        ::

            >>> pitch_segment = pitchtools.PitchSegment(
            ...     items=[0, 11, 7, 4, 2, 9, 3, 8, 10, 1, 5, 6],
            ...     )
            >>> numbered_interval_class_vector = pitchtools.IntervalClassVector(
            ...     items=pitch_segment,
            ...     item_class=pitchtools.NumberedInversionEquivalentIntervalClass,
            ...     )

        ::

            >>> items = sorted(numbered_interval_class_vector.items())
            >>> for interval, count in items:
            ...     print(interval, count)
            ...
            1 12
            2 12
            3 12
            4 12
            5 12
            6 6

    '''

    ### INITIALIZER ###

    def __init__(self, items=None, item_class=None):
        from abjad.tools import pitchtools
        prototype = (
            pitchtools.PitchSegment,
            pitchtools.PitchSet,
            pitchtools.PitchClassSegment,
            pitchtools.PitchClassSet,
            )
        if isinstance(items, prototype):
            intervals = []
            items = tuple(items)
            enumeration = sequencetools.Enumeration(items)
            pairs = enumeration.yield_pairs()
            for first, second in pairs:
                intervals.append(second - first)
            items = intervals
        Vector.__init__(
            self,
            items=items,
            item_class=item_class,
            )

    ### SPECIAL METHODS ###

    def __repr__(self):
        r'''Gets interpreter representation of interval-class vector.

        ..  container:: example

            Gets interpreter representation of interval-class vector:

            ::

                >>> segment = pitchtools.PitchSegment(
                ...     items=[0, 11, 7, 4, 2, 9, 3, 8, 10, 1, 5, 6],
                ...     )
                >>> vector = pitchtools.IntervalClassVector(
                ...     items=pitch_segment,
                ...     item_class=pitchtools.NumberedInversionEquivalentIntervalClass,
                ...     )

            ::

                >>> vector
                IntervalClassVector({1: 12, 2: 12, 3: 12, 4: 12, 5: 12, 6: 6}, item_class=NumberedInversionEquivalentIntervalClass)

        ..  container:: example

            Initializes from interpreter representation of interval-class
            vector:

            ::

                >>> pitchtools.IntervalClassVector(vector)
                IntervalClassVector({1: 12, 2: 12, 3: 12, 4: 12, 5: 12, 6: 6}, item_class=NumberedInversionEquivalentIntervalClass)

        Returns string.
        '''
        superclass = super(IntervalClassVector, self)
        return superclass.__repr__()

    ### PRIVATE PROPERTIES ###

    @property
    def _label(self):
        counts = []
        for i in range(7):
            counts.append(self[i])
        counts = ''.join([str(x) for x in counts])
        if len(self) == 13:
            quartertones = []
            for i in range(6):
                quartertones.append(self[i + 0.5])
            quartertones = ''.join([str(x) for x in quartertones])
            return r'\tiny \column { "%s" "%s" }' % (counts, quartertones)
        else:
            return r'\tiny %s' % counts

    @property
    def _named_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NamedIntervalClass

    @property
    def _numbered_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NumberedIntervalClass

    @property
    def _parent_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.IntervalClass

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(
        class_,
        selection,
        item_class=None,
        ):
        r'''Makes interval-class vector from `selection`.

        ..  container:: example

            Makes numbered inversion-equivalent interval-class vector from
            selection:

            ::

                >>> vector = pitchtools.IntervalClassVector.from_selection(
                ...     Chord("<c' d' b''>4"),
                ...     item_class=pitchtools.NumberedInversionEquivalentIntervalClass,
                ...     )
                >>> vector
                IntervalClassVector({1: 1, 2: 1, 3: 1}, item_class=NumberedInversionEquivalentIntervalClass)

        ..  container:: example

            Makes numbered interval-class vector from selection:

            ::

                >>> vector = pitchtools.IntervalClassVector.from_selection(
                ...     Chord("<c' d' b''>4"),
                ...     item_class=pitchtools.NumberedIntervalClass,
                ...     )
                >>> vector
                IntervalClassVector({-11: 1, -9: 1, -2: 1}, item_class=NumberedIntervalClass)

            .. todo:: This should probabaly be checked. Resulting values
                should probabaly be positive (or signless) instead of negative.

        ..  container:: example

            Makes named interval-class vector from selection:

            ::

                >>> vector = pitchtools.IntervalClassVector.from_selection(
                ...     Chord("<c' d' b''>4"),
                ...     item_class=None,
                ...     )
                >>> vector
                IntervalClassVector({'-M2': 1, '-M6': 1, '-M7': 1}, item_class=NamedIntervalClass)

            .. todo:: This should probabaly be checked. Resulting values
                should probabaly be positive (or signless) instead of negative.

        Returns new interval-class vector.
        '''
        from abjad.tools import pitchtools
        pitch_segment = pitchtools.PitchSegment.from_selection(selection)
        return class_(
            pitch_segment,
            item_class=item_class,
            )



# -*- coding: utf-8 -*-
from abjad.tools.datastructuretools.TypedList import TypedList
from abjad.tools.topleveltools import new


class InstrumentList(TypedList):
    r'''Instrument list.

    ..  container:: example

        ::

            >>> instruments = instrumenttools.InstrumentList([
            ...     instrumenttools.Flute(),
            ...     instrumenttools.Guitar()
            ...     ])

        ::

            >>> f(instruments)
            instrumenttools.InstrumentList(
                [
                    instrumenttools.Flute(
                        instrument_name='flute',
                        short_instrument_name='fl.',
                        instrument_name_markup=markuptools.Markup(
                            contents=['Flute'],
                            ),
                        short_instrument_name_markup=markuptools.Markup(
                            contents=['Fl.'],
                            ),
                        allowable_clefs=instrumenttools.ClefList(
                            [
                                indicatortools.Clef(
                                    name='treble',
                                    ),
                                ]
                            ),
                        pitch_range=pitchtools.PitchRange(
                            range_string='[C4, D7]',
                            ),
                        sounding_pitch_of_written_middle_c=pitchtools.NamedPitch("c'"),
                        ),
                    instrumenttools.Guitar(
                        instrument_name='guitar',
                        short_instrument_name='gt.',
                        instrument_name_markup=markuptools.Markup(
                            contents=['Guitar'],
                            ),
                        short_instrument_name_markup=markuptools.Markup(
                            contents=['Gt.'],
                            ),
                        allowable_clefs=instrumenttools.ClefList(
                            [
                                indicatortools.Clef(
                                    name='treble',
                                    ),
                                ]
                            ),
                        default_tuning=indicatortools.Tuning(
                            pitches=pitchtools.PitchSegment(
                                (
                                    pitchtools.NamedPitch('e,'),
                                    pitchtools.NamedPitch('a,'),
                                    pitchtools.NamedPitch('d'),
                                    pitchtools.NamedPitch('g'),
                                    pitchtools.NamedPitch('b'),
                                    pitchtools.NamedPitch("e'"),
                                    ),
                                item_class=pitchtools.NamedPitch,
                                ),
                            ),
                        pitch_range=pitchtools.PitchRange(
                            range_string='[E2, E5]',
                            ),
                        sounding_pitch_of_written_middle_c=pitchtools.NamedPitch('c'),
                        ),
                    ]
                )

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats instruments.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        superclass = super(InstrumentList, self)
        return superclass.__format__(format_specification=format_specification)

    def __repr__(self):
        r'''Gets interpreter representation of instrument list.

        ..  container:: example

            ::

                >>> instruments
                InstrumentList([Flute(), Guitar()])

        Returns string.
        '''
        contents = [repr(x) for x in self]
        contents = ', '.join(contents)
        return '{}([{}])'.format(type(self).__name__, contents)

    ### PRIVATE METHODS ###

    @staticmethod
    def _change_instrument_name_to_instrument(instrument_name):
        from abjad.tools import instrumenttools
        if instrument_name in (
            'alto',
            'baritone',
            'bass',
            'soprano',
            'tenor',
            ):
            instrument_name = instrument_name + ' Voice'
        instrument_name = instrument_name.title()
        instrument_name = instrument_name.replace(' ', '')
        instrument_name = instrument_name.replace('-', '')
        instrument_class = instrumenttools.__dict__[instrument_name]
        instrument = instrument_class()
        return instrument

    @staticmethod
    def _name_percussion(instrument, session):
        from abjad.tools import instrumenttools
        from ide import idetools
        if isinstance(instrument, instrumenttools.Percussion):
            Percussion = instrumenttools.Percussion
            items = Percussion.known_percussion[:]
            selector = idetools.Selector(session=session, items=items)
            instrument_name = selector._run()
            if selector._session.is_backtracking or instrument_name is None:
                return
            instrument = new(
                instrument,
                instrument_name=instrument_name,
                short_instrument_name=instrument_name,
                )
        return instrument

    ### ITEM CREATOR ###

    @staticmethod
    def _make_item_creator_class():
        from ide.idetools.Controller import Controller
        class ItemCreator(Controller):
            ### CLASS VARIABLES ###
            __slots__ = ('_is_ranged', '_target',)
            ### INITIALIZER ###
            def __init__(self, is_ranged=False, session=None, target=None):
                Controller.__init__(self, session=session)
                self._is_ranged = is_ranged
                self._target = target
            ### PRIVATE METHODS ###
            def _run(self):
                from abjad.tools import instrumenttools
                from ide import idetools
                controller = idetools.ControllerContext(controller=self)
                with controller:
                    items = instrumenttools.Instrument._list_instrument_names()
                    selector = idetools.Selector(
                        session=self._session,
                        items=items,
                        is_ranged=self._is_ranged,
                        )
                    result = selector._run()
                    if self._session.is_backtracking or not result:
                        return
                    if isinstance(result, list):
                        instrument_names = result
                    else:
                        instrument_names = [result]
                    instruments = []
                    class_ = InstrumentList
                    to_instrument = \
                        class_._change_instrument_name_to_instrument
                    name_percussion = class_._name_percussion
                    for instrument_name in instrument_names:
                        instrument = to_instrument(instrument_name)
                        instrument = name_percussion(instrument, self._session)
                        if instrument is not None:
                            instruments.append(instrument)
                    if self._is_ranged:
                        result = instruments[:]
                    else:
                        result = instruments[0]
                    self._target = result
                    return self.target
            @property
            def target(self):
                return self._target
        return ItemCreator

    ### PRIVATE PROPERTIES ###

    @property
    def _item_creator_class(self):
        item_creator_class = self._make_item_creator_class()
        return item_creator_class

    @property
    def _item_creator_class_kwargs(self):
        return {'is_ranged': True}



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Container___setitem___01():
    r'''Replaces in-score leaf with out-of-score leaf.
    '''

    voice = Voice("c'8 [ d'8 ] e'8 f'8")
    leaves = iterate(voice).by_leaf()
    attach(Glissando(), list(leaves))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [ \glissando
            d'8 ] \glissando
            e'8 \glissando
            f'8
        }
        '''
        )

    voice[1] = Note("c''8")

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [ \glissando
            c''8 ] \glissando
            e'8 \glissando
            f'8
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container___setitem___02():
    r'''Replaces in-score leaf with out-of-score container.
    '''

    voice = Voice("c'8 [ d'8 ] e'8 f'8")
    leaves = iterate(voice).by_leaf()
    glissando = Glissando(allow_repeat_pitches=True)
    attach(glissando, list(leaves))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [ \glissando
            d'8 ] \glissando
            e'8 \glissando
            f'8
        }
        '''
        )

    voice[1] = Container("c'16 c'16 c'16")

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [ \glissando
            {
                c'16 \glissando
                c'16 \glissando
                c'16 ] \glissando
            }
            e'8 \glissando
            f'8
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container___setitem___03():
    r'''Replaces in-score container with out-of-score leaf.
    '''

    voice = Voice("{ c'8 [ d'8 } { e'8 f'8 ] }")
    leaves = iterate(voice).by_leaf()
    attach(Glissando(), list(leaves))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ \glissando
                d'8 \glissando
            }
            {
                e'8 \glissando
                f'8 ]
            }
        }
        '''
        )

    voice[1] = Note("c''8")

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ \glissando
                d'8 \glissando
            }
            c''8 ]
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container___setitem___04():
    r'''Replaces in-score container with out-of-score tuplet.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 }")
    leaves = select(voice).by_leaf()
    attach(Beam(), leaves)
    attach(Glissando(), leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ \glissando
                d'8 \glissando
            }
            {
                e'8 \glissando
                f'8 ]
            }
        }
        '''
        )

    voice[1] = Tuplet(Multiplier(2, 3), "c'8 d'8 e'8")

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ \glissando
                d'8 \glissando
            }
            \times 2/3 {
                c'8 \glissando
                d'8 \glissando
                e'8 ]
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container___setitem___05():
    r'''Replaces in-score container with out-of-score leaf.
    '''

    voice = Voice("{ c'8 [ d'8 } { e'8 f'8 ] }")
    leaves = iterate(voice).by_leaf()
    attach(Glissando(), list(leaves))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ \glissando
                d'8 \glissando
            }
            {
                e'8 \glissando
                f'8 ]
            }
        }
        '''
        )

    voice[1] = Note("c''8")

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ \glissando
                d'8 \glissando
            }
            c''8 ]
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container___setitem___06():
    r'''Replaces in-score container with out-of-score leaf.
    '''

    voice = Voice(2 * Container("c'8 c'8 c'8 c'8"))
    voice = Voice("{ c'8 d'8 e'8 f'8 } { g'8 a'8 b'8 c''8 }")
    leaves = iterate(voice).by_leaf()
    attach(Beam(), list(leaves)[0:6])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [
                d'8
                e'8
                f'8
            }
            {
                g'8
                a'8 ]
                b'8
                c''8
            }
        }
        '''
        )

    voice[1] = Rest('r2')

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [
                d'8
                e'8
                f'8 ]
            }
            r2
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container___setitem___07():
    r'''Replaces note in one score with note from another score.
    '''

    notes = [
        Note("c'8"), Note("d'8"), Note("e'8"), 
        Note("f'8"), Note("g'8"), Note("a'8"),
        ]

    voice_1 = Voice(notes[:3])
    attach(Beam(), voice_1[:])

    assert format(voice_1) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8
            e'8 ]
        }
        '''
        )

    voice_2 = Voice(notes[3:])
    attach(Beam(), voice_2[:])

    assert format(voice_2) == stringtools.normalize(
        r'''
        \new Voice {
            f'8 [
            g'8
            a'8 ]
        }
        '''
        )

    voice_1[1] = voice_2[1]

    assert format(voice_1) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            g'8
            e'8 ]
        }
        '''
        )

    assert inspect_(voice_1).is_well_formed()

    assert format(voice_2) == stringtools.normalize(
        r'''
        \new Voice {
            f'8 [
            a'8 ]
        }
        '''
        )

    assert inspect_(voice_2).is_well_formed()


def test_scoretools_Container___setitem___08():
    r'''Replaces note in one score with container from another score.
    '''

    notes = [
        Note("c'8"), Note("d'8"), Note("e'8"),
        Note("f'8"), Note("g'8"), Note("a'8"), Note("b'8"),
        ]
    voice_1 = Voice(notes[:3])
    attach(Beam(), voice_1[:])

    assert format(voice_1) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8
            e'8 ]
        }
        '''
        )

    voice_2 = Voice(notes[3:])
    Container(voice_2[1:3])
    leaves = iterate(voice_2).by_leaf()
    attach(Glissando(), list(leaves))
    leaves = iterate(voice_2[1]).by_leaf()
    attach(Slur(), list(leaves))

    assert format(voice_2) == stringtools.normalize(
        r'''
        \new Voice {
            f'8 \glissando
            {
                g'8 \glissando (
                a'8 ) \glissando
            }
            b'8
        }
        '''
        )

    voice_1[1] = voice_2[1]

    assert format(voice_1) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            {
                g'8 (
                a'8 )
            }
            e'8 ]
        }
        '''
        )

    assert inspect_(voice_1).is_well_formed()

    assert format(voice_2) == stringtools.normalize(
        r'''
        \new Voice {
            f'8 \glissando
            b'8
        }
        '''
        )

    assert inspect_(voice_2).is_well_formed()


def test_scoretools_Container___setitem___09():
    r'''Sets leaf between unspanned components.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    staff[2:2] = [Note("g'8")]

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            d'8
            g'8
            e'8
            f'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container___setitem___10():
    r'''Sets leaf between spanned compoennts.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, staff[:])
    note = Note("g'8")
    staff[2:2] = [note]

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            d'8
            g'8
            e'8
            f'8 ]
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container___setitem___11():
    r'''Sets multiple leaves between spanned components.
    '''

    notes = [
        Note("c'8"), Note("d'8"), Note("e'8"),
        Note("f'8"), Note("g'8"), Note("a'8"),
        ]

    beginning = notes[:2]
    middle = notes[2:4]
    end = notes[4:]

    staff = Staff(beginning + end)
    beam = Beam()
    attach(beam, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            d'8
            g'8
            a'8 ]
        }
        '''
        )

    staff[2:2] = middle

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            d'8
            e'8
            f'8
            g'8
            a'8 ]
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container___setitem___12():
    r'''Replaces multiple spanned leaves with with single leaf.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, staff[:])
    note = Note("c''8")
    staff[1:3] = [note]

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            c''8
            f'8 ]
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container___setitem___13():
    r'''Replaces three spanned leaves with three different leaves.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, staff[:])
    notes = [Note("b'8"), Note("a'8"), Note("g'8")]
    staff[1:3] = notes

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            b'8
            a'8
            g'8
            f'8 ]
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container___setitem___14():
    r'''Replaces in-score container with contents of container.
    '''

    staff = Staff("{ c'8 [ d'8 } { e'8 f'8 ] }")

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8 ]
            }
        }
        '''
        )

    container = staff[0]
    staff[0:1] = container[:]

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            d'8
            {
                e'8
                f'8 ]
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()
    assert len(container) == 0


def test_scoretools_Container___setitem___15():
    r'''Sets first slice of staff equal to first element of first container in
    staff.
    '''

    staff = Staff("{ c'8 [ d'8 } { e'8 f'8 ] }")

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8 ]
            }
        }
        '''
        )

    staff[0:0] = staff[0][:1]

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            {
                d'8 [
            }
            {
                e'8
                f'8 ]
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container___setitem___16():
    r'''Sets first slice of staff equal to contents of first container in
    staff.

    Empties first container in staff.

    Leaves empty container in staff.
    '''

    staff = Staff("{ c'8 [ d'8 } { e'8 f'8 ] }")

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8 ]
            }
        }
        '''
        )

    staff[0:0] = staff[0][:]

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            d'8
            {
            }
            {
                e'8 [
                f'8 ]
            }
        }
        '''
        )


def test_scoretools_Container___setitem___17():
    r'''Set first slice of staff equal to contents of first container in staff;
    empties first container in staff.

    Sets contents of empty first container in staff equal to first component in
    second container in staff.
    '''

    staff = Staff("{ c'8 [ d'8 } { e'8 f'8 ] }")

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8 ]
            }
        }
        '''
        )

    container = staff[0]
    staff[0:0] = container[:]
    container[0:0] = staff[-1][:1]

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            d'8
            {
                e'8
            }
            {
                f'8 [ ]
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_scoretools_Container___setitem___18():
    r'''Extremely small coequal indices indicate first slice in staff.
    '''

    voice = Voice("c'8 [ d'8 e'8 f'8 ]")
    voice[-1000:-1000] = [Rest('r8')]

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            r8
            c'8 [
            d'8
            e'8
            f'8 ]
        }
        '''
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            r8
            c'8 [
            d'8
            e'8
            f'8 ]
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container___setitem___19():
    r'''Extremely large coequal indices indicate last slice in staff.
    '''

    voice = Voice("c'8 [ d'8 e'8 f'8 ]")
    voice[1000:1000] = [Rest('r8')]

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8
            e'8
            f'8 ]
            r8
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container___setitem___20():
    r'''You can use setitem to empty the contents of a container.
    When you do this, emptied components withdraw
    from absolutely all spanners.

    On the other hand, if you want to empty a container and
    allow the emptied components to remain embedded within spanners,
    use del(container) instead.
    '''

    staff = Staff("c'8 d'8 [ e'8 ] f'8")
    inner_container = Container(staff[1:3])
    outer_container = Container([inner_container])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            {
                {
                    d'8 [
                    e'8 ]
                }
            }
            f'8
        }
        '''
        )

    # sets contents of outer container to nothing
    outer_container[:] = []

    # outer container is empty and remains in score
    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            {
            }
            f'8
        }
        '''
        )

    # inner container leaves are no longer spanned
    assert format(inner_container) == stringtools.normalize(
        r'''
        {
            d'8
            e'8
        }
        '''
        )

    # ALTERNATIVE: use del(container)
    staff = Staff("c'8 d'8 [ e'8 ] f'8")
    inner_container = Container(staff[1:3])
    outer_container = Container([inner_container])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            {
                {
                    d'8 [
                    e'8 ]
                }
            }
            f'8
        }
        '''
        )

    # deletes outer container
    del(outer_container[:])

    # outer container is empty and remains in score (as before)
    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            {
            }
            f'8
        }
        '''
        )

    # inner container leaves are still spanned
    assert format(inner_container) == stringtools.normalize(
        r'''
        {
            d'8 [
            e'8 ]
        }
        '''
        )


def test_scoretools_Container___setitem___21():
    r'''Extends beam as leaves append and insert into staff.
    '''

    staff = Staff("c'8 [ { d'8 e'8 } f'8 ]")
    beam = inspect_(staff[0]).get_spanner(prototype=Beam)

    leaves = iterate(staff).by_leaf()
    assert beam.components == list(leaves)

    staff[1].append("g'8")
    leaves = iterate(staff).by_leaf()
    assert beam.components == list(leaves)

    staff[1].insert(0, "b'8")
    leaves = iterate(staff).by_leaf()
    assert beam.components == list(leaves)

    staff.insert(1, "a'8")
    leaves = iterate(staff).by_leaf()
    assert beam.components == list(leaves)

    staff.insert(3, "fs'8")
    leaves = iterate(staff).by_leaf()
    assert beam.components == list(leaves)

    assert format(staff) == systemtools.TestManager.clean_string(
        r'''
        \new Staff {
            c'8 [
            a'8
            {
                b'8
                d'8
                e'8
                g'8
            }
            fs'8
            f'8 ]
        }
        ''',
        )



# -*- coding: utf-8 -*-
from abjad.tools import datastructuretools
from abjad.tools import mathtools
from abjad.tools import metertools
from abjad.tools import scoretools
from abjad.tools import selectiontools
from abjad.tools import spannertools
from abjad.tools.topleveltools import attach
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker


class NoteRhythmMaker(RhythmMaker):
    r'''Note rhythm-maker.

    ..  container:: example

        Makes notes equal to the duration of input divisions. Adds ties where
        necessary:

        ::

            >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker()

        ::

            >>> divisions = [(5, 8), (3, 8)]
            >>> selections = rhythm_maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     selections,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> f(lilypond_file[Staff])
            \new RhythmicStaff {
                {
                    \time 5/8
                    c'2 ~
                    c'8
                }
                {
                    \time 3/8
                    c'4.
                }
            }

    Usage follows the two-step configure-once / call-repeatedly pattern shown
    here.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Rhythm-makers'

    __slots__ = (
        '_burnish_specifier',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        beam_specifier=None,
        burnish_specifier=None,
        division_masks=None,
        duration_spelling_specifier=None,
        logical_tie_masks=None,
        tie_specifier=None,
        tuplet_spelling_specifier=None,
        ):
        from abjad.tools import rhythmmakertools
        RhythmMaker.__init__(
            self,
            beam_specifier=beam_specifier,
            duration_spelling_specifier=duration_spelling_specifier,
            division_masks=division_masks,
            logical_tie_masks=logical_tie_masks,
            tie_specifier=tie_specifier,
            tuplet_spelling_specifier=tuplet_spelling_specifier,
            )
        if burnish_specifier is not None:
            prototype = rhythmmakertools.BurnishSpecifier
            assert isinstance(burnish_specifier, prototype)
        self._burnish_specifier = burnish_specifier

    ### SPECIAL METHODS ###

    def __call__(self, divisions, rotation=None):
        r'''Calls note rhythm-maker on `divisions`.

        ..  container:: example

            Calls rhythm-maker on divisions:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker()
                >>> divisions = [(5, 8), (3, 8)]
                >>> result = rhythm_maker(divisions)
                >>> for x in result:
                ...     x
                Selection([Note("c'2"), Note("c'8")])
                Selection([Note("c'4.")])

        Returns list of selections. Each selection holds one or more notes.
        '''
        return RhythmMaker.__call__(
            self,
            divisions,
            rotation=rotation,
            )

    def __format__(self, format_specification=''):
        r'''Formats note rhythm-maker.

        Set `format_specification` to `''` or `'storage'`.

        ..  container:: example

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker()
                >>> f(rhythm_maker)
                rhythmmakertools.NoteRhythmMaker()

        Returns string.
        '''
        superclass = super(NoteRhythmMaker, self)
        return superclass.__format__(format_specification=format_specification)

    def __repr__(self):
        r'''Gets interpreter representation.

        ..  container:: example

            ::

                >>> rhythmmakertools.NoteRhythmMaker()
                NoteRhythmMaker()

        Returns string.
        '''
        return super(NoteRhythmMaker, self).__repr__()

    ### PRIVATE METHODS ###

    def _apply_burnish_specifier(self, selections):
        if self.burnish_specifier is None:
            return selections
        elif self.burnish_specifier.outer_divisions_only:
            selections = self._burnish_outer_divisions(selections)
        else:
            selections = self._burnish_each_division(selections)
        return selections

    def _burnish_each_division(self, selections):
        message = 'NoteRhythmMaker does not yet implement'
        message += ' burnishing each division.'
        raise NotImplementedError(message)

    def _burnish_outer_divisions(self, selections):
        left_classes = self.burnish_specifier.left_classes
        left_counts = self.burnish_specifier.left_counts
        right_classes = self.burnish_specifier.right_classes
        right_counts = self.burnish_specifier.right_counts
        if left_counts:
            assert len(left_counts) == 1, repr(left_counts)
            left_count = left_counts[0]
        else:
            left_count = 0
        if right_counts:
            assert len(right_counts) == 1, repr(right_counts)
            right_count = right_counts[0]
        else:
            right_count = 0
        if left_count + right_count <= len(selections):
            middle_count = len(selections) - (left_count + right_count)
        elif left_count <= len(selections):
            right_count = len(selections) - left_count
            middle_count = 0
        else:
            left_count = len(selections)
            right_count = 0
            middle_count = 0
        assert left_count + middle_count + right_count == len(selections)
        new_selections = []
        left_classes = datastructuretools.CyclicTuple(left_classes)
        for i, selection in enumerate(selections[:left_count]):
            target_class = left_classes[i]
            new_selection = self._cast_selection(selection, target_class)
            new_selections.append(new_selection)
        if right_count:
            for selection in selections[left_count:-right_count]:
                new_selections.append(selection)
            right_classes = datastructuretools.CyclicTuple(right_classes)
            for i, selection in enumerate(selections[-right_count:]):
                target_class = right_classes[i]
                new_selection = self._cast_selection(selection, target_class)
                new_selections.append(new_selection)
        else:
            for selection in selections[left_count:]:
                new_selections.append(selection)
        return new_selections

    def _cast_selection(self, selection, target_class):
        new_selection = []
        for leaf in selection:
            new_leaf = target_class(leaf)
            new_selection.append(new_leaf)
        new_selection = selectiontools.Selection(new_selection)
        return new_selection

    def _make_music(self, divisions, rotation):
        from abjad.tools import rhythmmakertools
        selections = []
        duration_specifier = self._get_duration_spelling_specifier()
        tie_specifier = self._get_tie_specifier()
        tuplet_specifier = self._get_tuplet_spelling_specifier()
        for division in divisions:
            if (duration_specifier.spell_metrically == True or
                (duration_specifier.spell_metrically == 'unassignable' and
                not mathtools.is_assignable_integer(division.numerator))):
                meter = metertools.Meter(division)
                rhythm_tree_container = meter.root_node
                durations = [_.duration for _ in rhythm_tree_container]
            elif isinstance(duration_specifier.spell_metrically,
                rhythmmakertools.PartitionTable):
                partition_table = duration_specifier.spell_metrically
                durations = partition_table.respell_division(division)
            else:
                durations = [division]
            selection = scoretools.make_leaves(
                pitches=0,
                durations=durations,
                decrease_durations_monotonically=\
                    duration_specifier.decrease_durations_monotonically,
                forbidden_written_duration=\
                    duration_specifier.forbidden_written_duration,
                is_diminution=tuplet_specifier.is_diminution,
                use_messiaen_style_ties=tie_specifier.use_messiaen_style_ties,
                )
            if (
                1 < len(selection) and
                not selection[0]._has_spanner(spannertools.Tie)
                ):
                tie = spannertools.Tie(
                    use_messiaen_style_ties=tie_specifier.use_messiaen_style_ties,
                    )
                attach(tie, selection[:])
            selections.append(selection)
        selections = self._apply_burnish_specifier(selections)
        beam_specifier = self._get_beam_specifier()
        beam_specifier(selections)
        selections = self._apply_division_masks(selections, rotation)
        if duration_specifier.rewrite_meter:
            selections = duration_specifier._rewrite_meter_(
                selections,
                divisions,
                use_messiaen_style_ties=tie_specifier.use_messiaen_style_ties,
                )
        return selections

    ### PUBLIC PROPERTIES ###

    @property
    def beam_specifier(self):
        r'''Gets beam specifier.

        ..  container:: example

            Beams each division:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     beam_specifier=rhythmmakertools.BeamSpecifier(
                ...         beam_each_division=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(5, 32), (5, 32)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 5/32
                        c'8 ~ [
                        c'32 ]
                    }
                    {
                        c'8 ~ [
                        c'32 ]
                    }
                }

        ..  container:: example

            Beams divisions together:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     beam_specifier=rhythmmakertools.BeamSpecifier(
                ...         beam_divisions_together=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(5, 32), (5, 32)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 5/32
                        \set stemLeftBeamCount = #0
                        \set stemRightBeamCount = #1
                        c'8 [ ~
                        \set stemLeftBeamCount = #3
                        \set stemRightBeamCount = #1
                        c'32
                    }
                    {
                        \set stemLeftBeamCount = #1
                        \set stemRightBeamCount = #1
                        c'8 ~
                        \set stemLeftBeamCount = #3
                        \set stemRightBeamCount = #0
                        c'32 ]
                    }
                }

        ..  container:: example

            Makes no beams:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     beam_specifier=rhythmmakertools.BeamSpecifier(
                ...         beam_divisions_together=False,
                ...         beam_each_division=False,
                ...         ),
                ...     )

            ::

                >>> divisions = [(5, 32), (5, 32)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 5/32
                        c'8 ~
                        c'32
                    }
                    {
                        c'8 ~
                        c'32
                    }
                }

        Returns beam specifier.
        '''
        superclass = super(NoteRhythmMaker, self)
        return superclass.beam_specifier

    @property
    def burnish_specifier(self):
        r'''Gets burnish specifier.

        ..  container:: example

            Burnishes nothing:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker()

            ::

                >>> divisions = [(5, 8), (2, 8), (2, 8), (5, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 5/8
                        c'2 ~
                        c'8
                    }
                    {
                        \time 2/8
                        c'4
                    }
                    {
                        c'4
                    }
                    {
                        \time 5/8
                        c'2 ~
                        c'8
                    }
                }

        ..  container:: example

            Forces leaves of first division to be rests:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     burnish_specifier=rhythmmakertools.BurnishSpecifier(
                ...         left_classes=[Rest],
                ...         left_counts=[1],
                ...         outer_divisions_only=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(5, 8), (2, 8), (2, 8), (5, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 5/8
                        r2
                        r8
                    }
                    {
                        \time 2/8
                        c'4
                    }
                    {
                        c'4
                    }
                    {
                        \time 5/8
                        c'2 ~
                        c'8
                    }
                }

        ..  container:: example

            Forces leaves of first two divisions to be rests:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     burnish_specifier=rhythmmakertools.BurnishSpecifier(
                ...         left_classes=[Rest],
                ...         left_counts=[2],
                ...         outer_divisions_only=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(5, 8), (2, 8), (2, 8), (5, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 5/8
                        r2
                        r8
                    }
                    {
                        \time 2/8
                        r4
                    }
                    {
                        c'4
                    }
                    {
                        \time 5/8
                        c'2 ~
                        c'8
                    }
                }

        ..  container:: example

            Forces leaves of first and last divisions to rests:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     burnish_specifier=rhythmmakertools.BurnishSpecifier(
                ...         left_classes=[Rest],
                ...         left_counts=[1],
                ...         right_classes=[Rest],
                ...         right_counts=[1],
                ...         outer_divisions_only=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(5, 8), (2, 8), (2, 8), (5, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 5/8
                        r2
                        r8
                    }
                    {
                        \time 2/8
                        c'4
                    }
                    {
                        c'4
                    }
                    {
                        \time 5/8
                        r2
                        r8
                    }
                }

        ..  note:: Currently only works when `outer_divisions_only` is true.

        Returns burnish specifier or none.
        '''
        return self._burnish_specifier

    @property
    def division_masks(self):
        r'''Gets division masks.

        ..  container:: example

            No division masks:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker()

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        c'2
                    }
                    {
                        \time 3/8
                        c'4.
                    }
                    {
                        \time 4/8
                        c'2
                    }
                    {
                        \time 3/8
                        c'4.
                    }
                }

        ..  container:: example

            Silences every other division:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     division_masks=[
                ...         rhythmmakertools.SilenceMask(
                ...             pattern=patterntools.select_every([0], period=2),
                ...             ),
                ...         ],
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        r2
                    }
                    {
                        \time 3/8
                        c'4.
                    }
                    {
                        \time 4/8
                        r2
                    }
                    {
                        \time 3/8
                        c'4.
                    }
                }

        ..  container:: example

            Silences every output division:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     division_masks=[rhythmmakertools.silence_all()],
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        r2
                    }
                    {
                        \time 3/8
                        r4.
                    }
                    {
                        \time 4/8
                        r2
                    }
                    {
                        \time 3/8
                        r4.
                    }
                }

        ..  container:: example

            Silences every output division and uses multimeasure rests:

            ::

                >>> mask = rhythmmakertools.SilenceMask(
                ...     pattern=patterntools.select_all(),
                ...     use_multimeasure_rests=True,
                ...     )
                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     division_masks=[mask],
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        R1 * 1/2
                    }
                    {
                        \time 3/8
                        R1 * 3/8
                    }
                    {
                        \time 4/8
                        R1 * 1/2
                    }
                    {
                        \time 3/8
                        R1 * 3/8
                    }
                }

        ..  container:: example

            Silences every other output division except for the first and last:

            ::

                >>> pattern_1 = patterntools.select_every([0], period=2)
                >>> pattern_2 = patterntools.select([0, -1])
                >>> pattern = pattern_1 & ~pattern_2
                >>> mask = rhythmmakertools.SilenceMask(
                ...     pattern=pattern,
                ...     )
                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     division_masks=[mask],
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8), (2, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        c'2
                    }
                    {
                        \time 3/8
                        c'4.
                    }
                    {
                        \time 4/8
                        r2
                    }
                    {
                        \time 3/8
                        c'4.
                    }
                    {
                        \time 2/8
                        c'4
                    }
                }

        Set to masks or none.
        '''
        superclass = super(NoteRhythmMaker, self)
        return superclass.division_masks

    @property
    def duration_spelling_specifier(self):
        r'''Gets duration spelling specifier.

        ..  container:: example

            Spells durations with the fewest number of glyphs:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker()

            ::

                >>> divisions = [(5, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 5/8
                        c'2 ~
                        c'8
                    }
                    {
                        \time 3/8
                        c'4.
                    }
                }

        ..  container:: example

            Forbids notes with written duration greater than or equal to
            ``1/2``:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     duration_spelling_specifier=rhythmmakertools.DurationSpellingSpecifier(
                ...         forbidden_written_duration=Duration(1, 2),
                ...         ),
                ...     )

            ::

                >>> divisions = [(5, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 5/8
                        c'4 ~
                        c'4 ~
                        c'8
                    }
                    {
                        \time 3/8
                        c'4.
                    }
                }

        ..  container:: example

            Spells all divisions metrically when `spell_metrically` is true:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     duration_spelling_specifier=rhythmmakertools.DurationSpellingSpecifier(
                ...         spell_metrically=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 4), (6, 16), (9, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/4
                        c'4 ~
                        c'4 ~
                        c'4
                    }
                    {
                        \time 6/16
                        c'8. ~ [
                        c'8. ]
                    }
                    {
                        \time 9/16
                        c'8. ~ [
                        c'8. ~
                        c'8. ]
                    }
                }

        ..  container:: example

            Spells only unassignable durations metrically when
            `spell_metrically` is ``'unassignable'``:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     duration_spelling_specifier=rhythmmakertools.DurationSpellingSpecifier(
                ...         spell_metrically='unassignable',
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 4), (6, 16), (9, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/4
                        c'2.
                    }
                    {
                        \time 6/16
                        c'4.
                    }
                    {
                        \time 9/16
                        c'8. ~ [
                        c'8. ~
                        c'8. ]
                    }
                }

            ``9/16`` is spelled metrically because it is unassignable.
            The other durations are spelled with the fewest number of symbols
            possible.

        ..  container:: example

            Spells durations with custom partition table:

            ::

                >>> partition_table = rhythmmakertools.PartitionTable([
                ...     (5, [3, 2]),
                ...     (9, [3, 3, 3]),
                ...     ])
                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     duration_spelling_specifier=rhythmmakertools.DurationSpellingSpecifier(
                ...         spell_metrically=partition_table,
                ...         ),
                ...     )

            ::

                >>> divisions = [(5, 16), (9, 16), (10, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 5/16
                        c'8. ~ [
                        c'8 ]
                    }
                    {
                        \time 9/16
                        c'8. ~ [
                        c'8. ~
                        c'8. ]
                    }
                    {
                        \time 10/16
                        c'4. ~
                        c'4
                    }
                }

        ..  container:: example

            Rewrites meter:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     duration_spelling_specifier=rhythmmakertools.DurationSpellingSpecifier(
                ...         rewrite_meter=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(3, 4), (6, 16), (9, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 3/4
                        c'2.
                    }
                    {
                        \time 6/16
                        c'4.
                    }
                    {
                        \time 9/16
                        c'4. ~
                        c'8.
                    }
                }

        Returns duration spelling specifier or none.
        '''
        return RhythmMaker.duration_spelling_specifier.fget(self)

    @property
    def logical_tie_masks(self):
        r'''Gets logical tie masks.

        ..  container:: example

            No logical tie masks:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker()

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        c'2
                    }
                    {
                        \time 3/8
                        c'4.
                    }
                    {
                        \time 4/8
                        c'2
                    }
                    {
                        \time 3/8
                        c'4.
                    }
                }

        ..  container:: example

            Silences every other logical tie:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     logical_tie_masks=rhythmmakertools.silence_every([0], period=2)
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        r2
                    }
                    {
                        \time 3/8
                        c'4.
                    }
                    {
                        \time 4/8
                        r2
                    }
                    {
                        \time 3/8
                        c'4.
                    }
                }

        ..  container:: example

            Silences all logical ties:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     logical_tie_masks=rhythmmakertools.silence_all(),
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        r2
                    }
                    {
                        \time 3/8
                        r4.
                    }
                    {
                        \time 4/8
                        r2
                    }
                    {
                        \time 3/8
                        r4.
                    }
                }

        Set to masks or none.
        '''
        superclass = super(NoteRhythmMaker, self)
        return superclass.logical_tie_masks
    
    @property
    def tie_specifier(self):
        r'''Gets tie specifier.

        ..  container:: example

            Does not tie across divisions:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     tie_specifier=rhythmmakertools.TieSpecifier(
                ...         tie_across_divisions=False,
                ...         ),
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        c'2
                    }
                    {
                        \time 3/8
                        c'4.
                    }
                    {
                        \time 4/8
                        c'2
                    }
                    {
                        \time 3/8
                        c'4.
                    }
                }

        ..  container:: example

            Ties across divisions:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     tie_specifier=rhythmmakertools.TieSpecifier(
                ...         tie_across_divisions=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        c'2 ~
                    }
                    {
                        \time 3/8
                        c'4. ~
                    }
                    {
                        \time 4/8
                        c'2 ~
                    }
                    {
                        \time 3/8
                        c'4.
                    }
                }

        ..  container:: example

            Patterns ties across divisions:

            ::

                >>> pattern = patterntools.Pattern(
                ...     indices=[0],
                ...     period=2,
                ...     )
                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     tie_specifier=rhythmmakertools.TieSpecifier(
                ...         tie_across_divisions=pattern,
                ...         ),
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (4, 8), (3, 8)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        c'2 ~
                    }
                    {
                        \time 3/8
                        c'4.
                    }
                    {
                        \time 4/8
                        c'2 ~
                    }
                    {
                        \time 3/8
                        c'4.
                    }
                }

        ..  container:: example

            Uses Messiaen-style ties:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     tie_specifier=rhythmmakertools.TieSpecifier(
                ...         tie_across_divisions=True,
                ...         use_messiaen_style_ties=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(4, 8), (3, 8), (9, 16), (5, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 4/8
                        c'2
                    }
                    {
                        \time 3/8
                        c'4. \repeatTie
                    }
                    {
                        \time 9/16
                        c'2 \repeatTie
                        c'16 \repeatTie
                    }
                    {
                        \time 5/16
                        c'4 \repeatTie
                        c'16 \repeatTie
                    }
                }

        ..  container:: example

            Strips all ties:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     tie_specifier=rhythmmakertools.TieSpecifier(
                ...         strip_ties=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(7, 16), (1, 4), (5, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 7/16
                        c'4..
                    }
                    {
                        \time 1/4
                        c'4
                    }
                    {
                        \time 5/16
                        c'4
                        c'16
                    }
                }

        ..  container:: example

            Spells durations metrically and then strips all ties:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     duration_spelling_specifier=rhythmmakertools.DurationSpellingSpecifier(
                ...         spell_metrically=True,
                ...         ),
                ...     tie_specifier=rhythmmakertools.TieSpecifier(
                ...         strip_ties=True,
                ...         ),
                ...     )

            ::

                >>> divisions = [(7, 16), (1, 4), (5, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 7/16
                        c'8. [
                        c'8
                        c'8 ]
                    }
                    {
                        \time 1/4
                        c'4
                    }
                    {
                        \time 5/16
                        c'8. [
                        c'8 ]
                    }
                }

        Returns tie specifier.
        '''
        superclass = super(NoteRhythmMaker, self)
        return superclass.tie_specifier

    @property
    def tuplet_spelling_specifier(self):
        r'''Gets tuplet spelling specifier.

        ..  container:: example

            Spells tuplets as diminutions:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker()

            ::

                >>> divisions = [(5, 14), (3, 7)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 5/14
                        \tweak edge-height #'(0.7 . 0)
                        \times 4/7 {
                            c'2 ~
                            c'8
                        }
                    }
                    {
                        \time 3/7
                        \tweak edge-height #'(0.7 . 0)
                        \times 4/7 {
                            c'2.
                        }
                    }
                }

        ..  container:: example

            Spells tuplets as augmentations:

            ::

                >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
                ...     tuplet_spelling_specifier=rhythmmakertools.TupletSpellingSpecifier(
                ...         is_diminution=False,
                ...         ),
                ...     )

            ::

                >>> divisions = [(5, 14), (3, 7)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 5/14
                        \tweak text #tuplet-number::calc-fraction-text
                        \tweak edge-height #'(0.7 . 0)
                        \times 8/7 {
                            c'4 ~
                            c'16
                        }
                    }
                    {
                        \time 3/7
                        \tweak text #tuplet-number::calc-fraction-text
                        \tweak edge-height #'(0.7 . 0)
                        \times 8/7 {
                            c'4.
                        }
                    }
                }

        Returns tuplet spelling specifier or none.
        '''
        superclass = super(NoteRhythmMaker, self)
        return superclass.tuplet_spelling_specifier



# -*- coding: utf-8 -*-


def format_input_lines_as_doc_string(input_lines):
    r'''Formats `input_lines` as doc string.

    Formats expressions intelligently.

    Treats blank lines intelligently.

    Captures hash-suffixed line output.

    Use when writing docstrings.

    Example skipped because docstring goes crazy on example input.
    '''
    from abjad import abjad_configuration
    tab = ' ' * abjad_configuration.get_tab_width()
    start = tab + tab + '>>> '
    lines = input_lines.split('\n')
    last_line_index = len(lines) - 1
    most = ''
    for i, line in enumerate(lines):
        if line == '':
            if i not in (0, last_line_index):
                print(tab + tab)
                print(tab + '::')
                print(tab + tab)
        elif line.startswith('f('):
            print(_replace_line_with_format(tab, most, line))
        elif line.endswith('###'):
            _handle_repr_line(tab, most, line)
            most += line + '\n'
        else:
            most += line + '\n'
            print(start + line)


def _handle_repr_line(tab, most_lines, line):
    header = 'from abjad import *\n'
    most_lines = header + most_lines
    exec(most_lines)
    line = line.replace('#', '')
    print(tab + tab + '>>> ' + line)
    exec('__x = %s' % line)
    if __x is not None:
        print(tab + tab + repr(__x))


def _replace_line_with_format(tab, most_lines, last_line):
    header = 'from abjad import *\n'
    most_lines = header + most_lines
    exec(most_lines)
    last_variable = last_line[2:-1]
    print(tab + tab + '>>> ' + 'f(%s)' % last_variable)
    exec(most_lines)
    exec('__x = %s.format' % last_variable)
    format_lines = __x.split('\n')
    format_lines = [x.replace('\t', tab) for x in format_lines]
    format_lines = [tab + tab + format_line for format_line in format_lines]
    format_str = '\n'.join(format_lines)
    return format_str



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_DuratedComplexBeam___copy___01():

    staff = Staff([
        Container("c'32 d'32 e'32"),
        Container("f'32 g'32 a'32"),
        Container("b'32 c'32")
        ])
    durations = [inspect_(x).get_duration() for x in staff]

    beam = spannertools.DuratedComplexBeam(
        durations=durations,
        span_beam_count=2,
        direction=Down,
        )
    leaves = select(staff).by_leaf()
    attach(beam, leaves)

    new_staff = mutate(staff).copy()
    new_leaves = select(new_staff).by_leaf()
    new_beam = inspect_(new_leaves[0]).get_spanner(Beam)

    assert format(staff) == format(new_staff)
    assert new_beam.durations == beam.durations
    assert new_beam.span_beam_count == beam.span_beam_count
    assert new_beam.isolated_nib_direction == beam.isolated_nib_direction
    assert new_beam.direction == beam.direction



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import durationtools
from abjad.tools.abctools import AbjadObject


class QTargetBeat(AbjadObject):
    r'''Representation of a single "beat" in a quantization target.

    ::

        >>> beatspan = (1, 8)
        >>> offset_in_ms = 1500
        >>> search_tree = quantizationtools.UnweightedSearchTree({3: None})
        >>> tempo = Tempo((1, 4), 56)

    ::

        >>> q_target_beat = quantizationtools.QTargetBeat(
        ...     beatspan=beatspan,
        ...     offset_in_ms=offset_in_ms,
        ...     search_tree=search_tree,
        ...     tempo=tempo,
        ...     )

    ::

        >>> print(format(q_target_beat))
        quantizationtools.QTargetBeat(
            beatspan=durationtools.Duration(1, 8),
            offset_in_ms=durationtools.Offset(1500, 1),
            search_tree=quantizationtools.UnweightedSearchTree(
                definition={   3: None,
                    },
                ),
            tempo=indicatortools.Tempo(
                reference_duration=durationtools.Duration(1, 4), 
                units_per_minute=56,
                ),
            )

    Not composer-safe.

    Used internally by ``Quantizer``.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_beatspan',
        '_distances',
        '_grouping',
        '_offset_in_ms',
        '_q_events',
        '_q_grid',
        '_q_grids',
        '_search_tree',
        '_tempo',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        beatspan=None,
        offset_in_ms=None,
        search_tree=None,
        tempo=None,
        ):
        from abjad.tools import quantizationtools

        beatspan = beatspan or durationtools.Duration(0)
        beatspan = durationtools.Duration(beatspan)
        offset_in_ms = offset_in_ms or durationtools.Duration(0)
        offset_in_ms = durationtools.Offset(offset_in_ms)

        if search_tree is None:
            search_tree = quantizationtools.UnweightedSearchTree()
        assert isinstance(search_tree, quantizationtools.SearchTree)
        tempo = tempo or indicatortools.Tempo(durationtools.Duration(1, 4), 60)
        #tempo = indicatortools.Tempo(tempo)
        if isinstance(tempo, tuple):
            tempo = indicatortools.Tempo(*tempo)
        assert not tempo.is_imprecise

        q_events = []
        q_grids = []

        self._beatspan = beatspan
        self._distances = {}
        self._offset_in_ms = offset_in_ms
        self._q_events = q_events
        self._q_grid = None
        self._q_grids = q_grids
        self._search_tree = search_tree
        self._tempo = tempo

    ### SPECIAL METHODS ###

    def __call__(self, job_id):
        r'''Calls q-target beat.

        Returns quantization job.
        '''
        from abjad.tools import quantizationtools
        if not self.q_events:
            return None
        assert all(isinstance(x, quantizationtools.QEvent)
            for x in self.q_events)
        q_event_proxies = []
        for q_event in self.q_events:
            q_event_proxy = quantizationtools.QEventProxy(
                q_event,
                self.offset_in_ms,
                self.offset_in_ms + self.duration_in_ms,
                )
            q_event_proxies.append(q_event_proxy)
        return quantizationtools.QuantizationJob(
            job_id, self.search_tree, q_event_proxies)

    def __format__(self, format_specification=''):
        r'''Formats q-event.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    ### PUBLIC PROPERTIES ###

    @property
    def beatspan(self):
        r'''Beatspan of q-target beat.

        ::

            >>> q_target_beat.beatspan
            Duration(1, 8)

        Returns duration.
        '''
        return self._beatspan

    @property
    def distances(self):
        r'''A list of computed distances between the ``QEventProxies``
        associated with a ``QTargetBeat`` instance, and each ``QGrid``
        generated for that beat.

        Used internally by the ``Quantizer``.

        Returns tuple.
        '''
        return self._distances

    @property
    def duration_in_ms(self):
        r'''Duration in milliseconds of the q-targeg beat.

        ::

            >>> q_target_beat.duration_in_ms
            Duration(3750, 7)

        Returns duration.
        '''
        from abjad.tools import quantizationtools
        return self.tempo.duration_to_milliseconds(self.beatspan)

    @property
    def offset_in_ms(self):
        r'''Offset in milliseconds of q-target beat.

        ::

            >>> q_target_beat.offset_in_ms
            Offset(1500, 1)

        Returns offset.
        '''
        return self._offset_in_ms

    @property
    def q_events(self):
        r'''A list for storing ``QEventProxy`` instances.

        Used internally by the ``Quantizer``.

        Returns list.
        '''
        return self._q_events

    @property
    def q_grid(self):
        r'''The ``QGrid`` instance selected by a ``Heuristic``.

        Used internally by the ``Quantizer``.

        Returns ``QGrid`` instance.
        '''
        return self._q_grid

    @property
    def q_grids(self):
        r'''A tuple of ``QGrids`` generated by a ``QuantizationJob``.

        Used internally by the ``Quantizer``.

        Returns tuple.
        '''
        return self._q_grids

    @property
    def search_tree(self):
        r'''Search tree of q-target beat.

        ::

            >>> q_target_beat.search_tree
            UnweightedSearchTree(definition={3: None})

        Returns search tree.
        '''
        return self._search_tree

    @property
    def tempo(self):
        r'''Tempo of q-target beat.

        ::

            >>> q_target_beat.tempo
            Tempo(reference_duration=Duration(1, 4), units_per_minute=56)

        Returns tempo.
        '''
        return self._tempo



# -*- coding: utf-8 -*-
import copy
from abjad import *


def test_scoretools_Staff___copy___01():
    r'''Staves (shallow) copy grob overrides and context settings
    but not musical content.
    '''

    staff_1 = Staff("c'8 d'8 e'8 f'8")
    override(staff_1).note_head.color = 'red'
    set_(staff_1).tuplet_full_length = True


    staff_2 = copy.copy(staff_1)

    assert format(staff_2) == stringtools.normalize(
        r'''
        \new Staff \with {
            \override NoteHead.color = #red
            tupletFullLength = ##t
        } {
        }
        '''
        )



# -*- coding: utf-8 -*-
import platform
import unittest
from abjad.tools import abjadbooktools
from abjad.tools import stringtools


@unittest.skipIf(
    platform.python_implementation() != 'CPython',
    'Only for CPython',
    )
class TestLaTeXDocumentHandler(unittest.TestCase):

    maxDiff = None

    def test_hide_1(self):
        input_file_contents = [
            '\\begin{comment}',
            '<abjad>[hide=true]',
            'def do_something(argument):',
            "    print('before')",
            '    print(argument + 1)',
            "    print('after')",
            '',
            '</abjad>',
            '\\end{comment}',
            '',
            '\\begin{comment}',
            '<abjad>',
            'do_something(23)',
            '</abjad>',
            '\\end{comment}',
            ]
        document_handler = abjadbooktools.LaTeXDocumentHandler()
        input_blocks = document_handler.collect_input_blocks(input_file_contents)
        input_blocks = tuple(input_blocks.values())
        assert input_blocks[0].code_block_specifier is not None
        assert input_blocks[0].code_block_specifier.hide
        assert input_blocks[1].code_block_specifier is None

    def test_hide_2(self):
        input_file_contents = [
            '\\begin{comment}',
            '<abjad>[hide=true]',
            'def do_something(argument):',
            "    print('before')",
            '    print(argument + 1)',
            "    print('after')",
            '',
            '</abjad>',
            '\\end{comment}',
            '',
            '\\begin{comment}',
            '<abjad>',
            'do_something(23)',
            '</abjad>',
            '\\end{comment}',
            ]
        document_handler = abjadbooktools.LaTeXDocumentHandler(
            input_file_contents=input_file_contents,
            )
        rebuilt_source = document_handler(return_source=True)
        assert rebuilt_source == stringtools.normalize(
            """
            \\begin{comment}
            <abjad>[hide=true]
            def do_something(argument):
                print('before')
                print(argument + 1)
                print('after')

            </abjad>
            \\end{comment}

            \\begin{comment}
            <abjad>
            do_something(23)
            </abjad>
            \\end{comment}

            %%% ABJADBOOK START %%%
            \\begin{lstlisting}
            >>> do_something(23)
            before
            24
            after
            \\end{lstlisting}
            %%% ABJADBOOK END %%%
            """,
            )
